@@ -24,6 +24,9 @@
 #if defined(HAVE_SYS_TYPES_H)
 #include <sys/types.h>
 #endif
+#if defined(HAVE_SYS_RESOURCE_H)
+#include <sys/resource.h>
+#endif
 #if defined(HAVE_WINNT_H)
 #include <winnt.h>
 #endif
@@ -39,14 +42,24 @@
 #if defined(HAVE_BSD_LIBC_H)
 #include <bsd/libc.h>
 #endif
+#if defined(HAVE_KERNEL_OS_H)
+#include <kernel/OS.h>
+#endif
 
 #undef	__NORETURN__
 #if defined(__GNUC__)
-#define	__NORETURN__ __attribute__((noreturn))
+#define	__NORETURN__ __attribute__((__noreturn__))
 #else
 #define	__NORETURN__
 #endif
 
+#undef	__UNUSED__
+#if defined(__GNUC__)
+#define	__UNUSED__ __attribute__((__unused__))
+#else
+#define	__UNUSED__
+#endif
+
 /* SunOS has on_exit only */
 #if !defined(HAVE_ATEXIT) && defined(HAVE_ON_EXIT)
 #define atexit(x)       on_exit(x, 0)
