@@ -15,28 +15,39 @@
 #include "config-std.h"
 #include "config-mem.h"
 #include "config-signal.h"
+#include "config-hacks.h"
 #include "gtypes.h"
 #include "access.h"
 #include "object.h"
 #include "constants.h"
-#include "classMethod.h"
+#include "errors.h"
 #include "baseClasses.h"
+#include "classMethod.h"
+#include "stringSupport.h"
 #include "thread.h"
 #include "lookup.h"
 #include "slots.h"
 #include "machine.h"
 #include "exception.h"
 #include "itypes.h"
+#include "gc.h"
 #include "md.h"
+#include "java_lang_Cloneable.h"
+#include "gcj/gcj.h"
+#include "xprofiler.h"
+#include "feedback.h"
+#include "debugFile.h"
+#include "fileSections.h"
 
 Utf8Const* init_name;
-Utf8Const* void_signature;
 Utf8Const* final_name;
+Utf8Const* void_signature;
 Utf8Const* constructor_name;
 Utf8Const* Code_name;
 Utf8Const* LineNumberTable_name;
 Utf8Const* ConstantValue_name;
 Utf8Const* Exceptions_name;
+Utf8Const* SourceFile_name;
 
 static Hjava_lang_Class dummyClassClass;
 Hjava_lang_Class* ClassClass = &dummyClassClass;
@@ -44,6 +55,8 @@ Hjava_lang_Class* StringClass;
 Hjava_lang_Class* ObjectClass;
 Hjava_lang_Class* SystemClass;
 Hjava_lang_Class* SerialClass;
+Hjava_lang_Class* CloneClass;
+Hjava_lang_Class* PtrClass;
 
 Hjava_lang_Class* javaLangVoidClass;
 Hjava_lang_Class* javaLangBooleanClass;
@@ -55,26 +68,33 @@ Hjava_lang_Class* javaLangLongClass;
 Hjava_lang_Class* javaLangFloatClass;
 Hjava_lang_Class* javaLangDoubleClass;
 
+Hjava_lang_Class* javaLangThrowable;
 Hjava_lang_Class* javaLangArrayIndexOutOfBoundsException;
 Hjava_lang_Class* javaLangNullPointerException;
+Hjava_lang_Class* javaLangArithmeticException;
+
+/* Let's not load this if we can't open Klasses.jar */
+Hjava_lang_Class* javaIoIOException;
 
 #define SYSTEMCLASS "java/lang/System"
 #define	SERIALCLASS "java/io/Serializable"
+#define	CLONECLASS  "java/lang/Cloneable"
+#define PTRCLASS    "kaffe/util/Ptr"
 
 #define	INIT			"<clinit>"
-#define	VOIDSIG			"()V"
 #define	FINAL			"finalize"
+#define	VOIDSIG			"()V"
 #define	CONSTRUCTOR_NAME	"<init>"
 
 /* Initialisation prototypes */
 void initClasspath(void);
-void initExceptions(void);
 void initNative(void);
 void initThreads(void);
-void initTypes(void);
 
 /*
  * Initialise the machine.
+ *
+ * XXX: fix the naming conventions at some point (initXXX vs. xxxInit)
  */
 void
 initialiseKaffe(void)
@@ -82,13 +102,34 @@ initialiseKaffe(void)
         /* Set default thread stack size if not set */
         extern JavaVMInitArgs Kaffe_JavaVMArgs[];
 	int threadStackSize;
-	Method* meth;
 
 	/* Machine specific initialisation first */
 #if defined(INIT_MD)
 	INIT_MD();
 #endif
 
+	/* Register allocation types with gc subsystem */
+	main_collector = initCollector();
+	GC_init(main_collector);
+
+#if defined(KAFFE_XPROFILER)
+	/* Start up the profiler here so we can cover init stuff */
+	if( xProfFlag )
+	{
+		if( !enableXProfiling() )
+		{
+			fprintf(stderr,
+				"Unable to initialize cross "
+				"language profiling\n");
+		}
+	}
+#endif
+#if defined(KAFFE_XDEBUGGING)
+	if( machine_debug_filename )
+	{
+		machine_debug_file = createDebugFile(machine_debug_filename);
+	}
+#endif
         threadStackSize = Kaffe_JavaVMArgs[0].nativeStackSize;
  
         if (threadStackSize == 0) {
@@ -96,7 +137,11 @@ initialiseKaffe(void)
         }
   
 	/* Initialise the (native) threading system */
-	(*Kaffe_ThreadInterface.init)(threadStackSize);
+	initNativeThreads(threadStackSize);
+
+	/* Initialise the string and utf8 systems */
+	stringInit();
+	utf8ConstInit();
 
 	/* Setup CLASSPATH */
 	initClasspath();
@@ -104,44 +149,83 @@ initialiseKaffe(void)
 	/* Init native support */
 	initNative();
 
+#if defined(KAFFE_FEEDBACK)
+	/* Install any file sections used by feedback */
+	installFileSections();
+	if( feedback_filename )
+	{
+		if( feedbackFile(feedback_filename) )
+		{
+			syncFeedback();
+			processFeedback();
+		}
+	}
+#endif
+	
 	/* Create the initialise and finalize names and signatures. */
-	init_name = makeUtf8ConstFixed(INIT, -1);
-	void_signature = makeUtf8ConstFixed(VOIDSIG, -1);
-	final_name = makeUtf8ConstFixed(FINAL, -1);
-	constructor_name = makeUtf8ConstFixed(CONSTRUCTOR_NAME, -1);
-	Code_name = makeUtf8ConstFixed("Code", -1);
-	LineNumberTable_name = makeUtf8ConstFixed("LineNumberTable", -1);
-	ConstantValue_name = makeUtf8ConstFixed("ConstantValue", -1);
-	Exceptions_name = makeUtf8ConstFixed("Exceptions", -1);
+	init_name = utf8ConstNew(INIT, -1);
+	final_name = utf8ConstNew(FINAL, -1);
+	void_signature = utf8ConstNew(VOIDSIG, -1);
+	constructor_name = utf8ConstNew(CONSTRUCTOR_NAME, -1);
+	Code_name = utf8ConstNew("Code", -1);
+	LineNumberTable_name = utf8ConstNew("LineNumberTable", -1);
+	ConstantValue_name = utf8ConstNew("ConstantValue", -1);
+	Exceptions_name = utf8ConstNew("Exceptions", -1);
+	SourceFile_name = utf8ConstNew("SourceFile", -1);
+
+	if (!(init_name && final_name && void_signature &&
+	      constructor_name && Code_name && LineNumberTable_name &&
+	      ConstantValue_name && Exceptions_name &&
+	      SourceFile_name)) {
+		fprintf(stderr, "not enough memory to run kaffe\n");
+		ABORT();
+	}
+
+#if defined(HAVE_GCJ_SUPPORT)
+	/* Init GCJ support */
+	gcjInit();
+#endif
 
 	/* Read in base classes */
 	initBaseClasses();
 
+#if defined(HAVE_GCJ_SUPPORT)
+	/* tell gcj where primitive classes are */
+	gcjInitPrimitiveClasses();
+#endif
+
 	/* Setup exceptions */
 	initExceptions();
 
+	/* Setup locking */
+	initLocking();
+
 	/* Init thread support */
 	initThreads();
 
-	/* Initialise the System */
-	do_execute_java_class_method("java.lang.System", "initializeSystemClass", "()V");
+	/* Now enable collector */
+	GC_enable(main_collector);
 }
 
 /*
  * We need to use certain classes in the internal machine so we better
  * get them in now in a known way so we can refer back to them.
- * Currently we need java/lang/Object, java/lang/Class, java/lang/String
- * and java/lang/System.
  */
 void
 initBaseClasses(void)
 {
+	Field *f;
+	errorInfo einfo;
+	Utf8Const *utf8;
+	extern char* realClassPath;
+
 	/* Primitive types */
 	initTypes();
 
 	/* The base types */
 	loadStaticClass(&ObjectClass, OBJECTCLASS);
 	loadStaticClass(&SerialClass, SERIALCLASS);
+	loadStaticClass(&CloneClass, CLONECLASS);
 	loadStaticClass(&ClassClass, CLASSCLASS);
 	loadStaticClass(&StringClass, STRINGCLASS);
 	loadStaticClass(&SystemClass, SYSTEMCLASS);
@@ -149,6 +233,7 @@ initBaseClasses(void)
 	/* We must to a little cross tidying */
 	ObjectClass->head.dtable = ClassClass->dtable;
 	SerialClass->head.dtable = ClassClass->dtable;
+	CloneClass->head.dtable = ClassClass->dtable;
 	ClassClass->head.dtable = ClassClass->dtable;
 
 	/* Basic types classes */
@@ -161,35 +246,57 @@ initBaseClasses(void)
 	loadStaticClass(&javaLangLongClass, "java/lang/Long");
 	loadStaticClass(&javaLangFloatClass, "java/lang/Float");
 	loadStaticClass(&javaLangDoubleClass, "java/lang/Double");
+	loadStaticClass(&PtrClass, PTRCLASS);
 
 	/* Exception handling types */
+	loadStaticClass(&javaLangThrowable, "java/lang/Throwable");
 	loadStaticClass(&javaLangArrayIndexOutOfBoundsException, "java/lang/ArrayIndexOutOfBoundsException");
 	loadStaticClass(&javaLangNullPointerException, "java/lang/NullPointerException");
+	loadStaticClass(&javaLangArithmeticException, "java/lang/ArithmeticException");
+	loadStaticClass(&javaIoIOException, "java/io/IOException");
 
 	/* Fixup primitive types */
 	finishTypes();
-}
+	processClass(StringClass, CSTATE_COMPLETE, &einfo);
 
-/*
- * Setup the internal exceptions.
- */
-void
-initExceptions(void)
-{
-DBG(INIT,	printf("initExceptions()\n");			)
-	if (DBGEXPR(EXCEPTION, false, true)) {
-	/* Catch signals we need to convert to exceptions */
-#if defined(SIGSEGV)
-		catchSignal(SIGSEGV, nullException);
-#endif
-#if defined(SIGBUS)
-		catchSignal(SIGBUS, nullException);
-#endif
-#if defined(SIGFPE)
-		catchSignal(SIGFPE, floatingException);
-#endif
-#if defined(SIGPIPE)
-		catchSignal(SIGPIPE, SIG_IGN);
-#endif
+	/* 
+	 * To make sure we have some ground to stand on, we doublecheck
+	 * that we really got Kaffe's java.lang.Cloneable class here.
+	 * Kaffe's class has a final public static field called 
+	 * "KAFFE_VERSION".
+	 */
+	utf8 = utf8ConstNew("KAFFE_VERSION", -1);
+	if (!utf8) {
+		fprintf(stderr, "not enough memory to run kaffe\n");
+		ABORT();
+	}
+	f = lookupClassField(CloneClass, utf8, true, &einfo);
+	utf8ConstRelease(utf8);
+	if (f == 0) {
+		fprintf(stderr,
+		    "\nCould not initialize Kaffe.\n"
+		    "It's likely that your CLASSPATH settings are wrong.  "
+		    "Please make sure\nyour CLASSPATH does not include any "
+		    "java.lang.* classes from other JVM\nvendors, such as "
+		    "Sun's classes.zip, BEFORE Kaffe's Klasses.jar.\n"
+		    "It is okay to have classes.zip AFTER Klasses.jar\n\n"
+		    "The current effective classpath is `%s'\n\n", 
+		    realClassPath
+		);
+		EXIT(-1);
+	}
+
+	if (*(jint*)FIELD_ADDRESS(f) != java_lang_Cloneable_KAFFE_VERSION) {
+		fprintf(stderr,
+		    "\nCould not initialize Kaffe.\n"
+		    "Your Klasses.jar version is %3.2f, but this VM "
+		    "was compiled with version %3.2f\n\n"
+		    "The current effective classpath is `%s'\n\n", 
+		    *(jint*)FIELD_ADDRESS(f)/100.0, 
+		    java_lang_Cloneable_KAFFE_VERSION/100.0,
+		    realClassPath
+		);
+		EXIT(-1);
 	}
 }
+
