@@ -1,3 +1,12 @@
+/*
+ * Copyright (c) 1998, 1999, 2000 The University of Utah. All rights reserved.
+ *
+ * See the file "license.terms" for information on usage and redistribution
+ * of this file.
+ *
+ * Contributed by the Flux Research Group at the University of Utah.
+ * Authors: Godmar Back, Patrick Tullmann
+ */
 /*
  * debug.c
  *
@@ -7,11 +16,6 @@
  * disabled in which case all of the code "just goes away",
  * or, if its enabled, the area of code to debug is dynamically
  * chosen at run time.
- *
- * INSERT COPYRIGHT HERE
- *
- * Written by Patrick Tullmann <tullmann@cs.utah.edu>, 1998
- *	   and Godmar Back <gback@cs.utah.edu>
  */
 
 #include "config.h"
@@ -31,14 +35,15 @@
 /* Don't waste space with the debugging functions */
 
 void dbgSetMask(jlong m) { }
-void dbgSetMaskStr(char *s) { }
+int dbgSetMaskStr(char *s) { return 0; }
 
 #else /* Actually define the functions */
 /* --- Debugging is enabled --- */ 
 
 jlong kaffevmDebugMask = DEFAULT_DEBUG_MASK;
 
-void dbgSetMask(jlong mask)
+void
+dbgSetMask(jlong mask)
 {
 	kaffevmDebugMask = mask;
 }
@@ -57,25 +62,30 @@ static struct debug_opts
 	/* XXX Clean these names up, re-order them and make them consistent. */
 #define D(name, str) { #name, DBG_ ## name, str }
 	D(NONE, "Nothing"),
-	D(VMLOCKS,  "Mutex lock/unlock operations called by VM"),
+	D(SLOWLOCKS,  "Heavy lock/unlock operations"),
 	D(VMCONDS,  "Show condition variable operations called by VM"),
 	D(NEWINSTR, "Show softnew_ instructions (NEW, NEWARRAY, ANEWARRAY)"),
 	D(VMTHREAD, "Show some java.lang.Thread operations called by VM"),
 	D(JTHREAD,  "Show jthread operations--jthreads only"),
 	D(JTHREADDETAIL,  "Show jthread operations (more detail)"),
 	D(JTHREADNOPREEMPT,  "Disable preemption in--jthreads only"),
-	D(DETECTDEADLOCK,  "Show when jthread system deadlocks--jthreads only"),
 	D(EXCEPTION, "Debug exceptions, don't catch traps"),
+	D(STACKTRACE, "Debug stack trace inspection"),
 	D(INIT,     "Show initialization steps."),
+	D(INITCLASSPATH,     "Show classpath initialization."),
 	D(BREAKONEXIT, "Cause an exception before exiting. (Useful under GDB)"),
 	D(GCPRIM,   "Debug gc_primitive_*"),
 	D(GCALLOC,   "Debug gc_heap_alloc*"),
 	D(GCFREE,   "Debug gc_heap_free*"),
 	D(GCSYSALLOC,   "Show allocations of system memory"),
-	D(GCSTAT,   "Show allocation statistics"),
+	D(GCWALK,   "Show gc walking"),
+	D(GCPRECISE, "Debug precise collection of the heap."),
+	D(GCSTAT,   "count instances of class and array types"),
+	D(GCDIAG,   "Perform diagnostic checks on heap" ),
 	{ "GCMEM", DBG_GCPRIM|DBG_GCALLOC|DBG_GCFREE|DBG_GCSYSALLOC|DBG_GCSTAT, 
 			"All allocation and free operations in gc-mem" },
 	D(SLACKANAL,   "Print internal fragmentation statistics."),
+
 	D(ASYNCSTDIO, "Make stdio fds asynchronous despite debugging."),
 	D(CATCHOUTOFMEM, "Catch recursive out of memory exceptions."),
 	D(JARFILES, "Debug reading JAR files in jar.c."),
@@ -85,22 +95,53 @@ static struct debug_opts
 	D(INT_RETURN, "Show return from function. (interpreter)"),
 	D(INT_VMCALL, "Show call to virtualMachine. (interpreter)"),
 	D(INT_CHECKS, "Show various checks. (interpreter)"),
-	D(ELOOKUP, "Debug exception lookup"),
-	D(FLOOKUP, "Debug field lookup"),
-	D(MLOOKUP, "Debug method lookup"),
+	D(ELOOKUP, "Debug exception lookup."),
+	D(FLOOKUP, "Debug field lookup."),
+	D(MLOOKUP, "Debug method lookup."),
+	D(CLASSLOOKUP, "Debug class lookup."),
 	D(JIT, 	"Debug JIT compiler--show emitted instructions."),
 	D(MOREJIT, 	"Debug JIT compiler--show callinfo."),
 	D(NOGC,	"Turn garbage collection off."),
+	D(STATICINIT,	"Announce when static initializers are called."),
+	D(CLASSFILE,	"Show when methods and fields are added."),
+	D(RESERROR,	"Show error in class file resolution."),
+	D(VMCLASSLOADER,"Announce when VM calls class loaders.."),
+	D(LOCKCONTENTION,"Show when a lock is contended."),
+	D(CODEANALYSE, "Debug code analysis/verification."),
+	D(CLASSGC, 	"Debug class garbage collection."),
+	D(NEWOBJECT, 	"Show when new objects are allocated."),
+	D(FINALIZE, 	"Debug finalization."),
+	D(NATIVELIB, 	"Debug native library operations."),
+	D(NATIVENET,	"Show network accesses from the native code."),
+
+	D(AWT_MEM,	"AWT subsystem."),
+	D(AWT_CLR,	"AWT subsystem."),
+	D(AWT_EVT,	"AWT subsystem."),
+	D(AWT_IMG,	"AWT subsystem."),
+	D(AWT_WND,	"AWT subsystem."),
+	D(AWT_GRA,	"AWT subsystem."),
+	D(AWT_FNT,	"AWT subsystem."),
+	D(AWT,		"Debug all of AWT."),
+
+	D(GCJ,		"Debug GCJ support."),
+	D(GCJMORE,	"Debug GCJ support (additional msg)."),
+
+	D(REGFORCE,	"Debug forced registers framework."),
 
 	/* you can define combinations too */
 	{ "lookup", DBG_MLOOKUP|DBG_ELOOKUP|DBG_FLOOKUP, 
 			"Various lookup operations" },
 
-	{ "thread", DBG_JTHREAD|DBG_VMLOCKS|DBG_VMCONDS, 
+	{ "gcj", DBG_GCJ|DBG_GCJMORE,
+			"Debug GCJ support in detailed" },
+	{ "thread", DBG_JTHREAD|DBG_SLOWLOCKS|DBG_VMCONDS, 
 			"Thread operations and locking operations" },
 
 	{ "intrp", DBG_INT_NATIVE|DBG_INT_RETURN|DBG_INT_VMCALL, 
 			"Calls of interpreter (without instructions)" },
+	{ "deadlock", DBG_LOCKCONTENTION, "Debug deadlocks" },
+	{ "classgc", DBG_CLASSGC|DBG_FINALIZE, 
+			"Debug finalization (show finalization)" },
 	{ "intrpA", 
 		DBG_INT_CHECKS|DBG_INT_INSTR|DBG_INT_NATIVE|
 		DBG_INT_RETURN|DBG_INT_VMCALL, 
@@ -119,7 +160,8 @@ void printDebugBuffer(void);
 
 #define NELEMS(a)	(sizeof (a) / sizeof(a[0]))
 
-void dbgSetMaskStr(char *mask_str)
+int
+dbgSetMaskStr(char *mask_str)
 {
 	int i;
 	char *separators = "|,";
@@ -131,17 +173,22 @@ void dbgSetMaskStr(char *mask_str)
 
 	if (!opt) {
 		kaffevmDebugMask = DEFAULT_DEBUG_MASK;
-		return;
+		return 1;
 	}
 
+#if defined(JTHREAD_RESTORE_FD)
+	jthreadRestoreFD(0);
+	jthreadRestoreFD(1);
+	jthreadRestoreFD(2);
+#endif
 	/* Special target 'list' lists all the defined options */
-	if (!strcmp(opt, "list")) {
-		printf("Available debug opts: \n");
-		printf("  %-15s\t%16s  %s\n", "Option", "Mask", "Description");
+	if (!strcasecmp(opt, "list")) {
+		dprintf("Available debug opts: \n");
+		dprintf("  %-15s\t%16s  %s\n", "Option", "Mask", "Description");
 		for (i = 0; i < NELEMS(debug_opts); i++)
 			if (debug_opts[i].mask>>32)
 			{
-				printf("  %-15s\t%8X%08X  %s\n", 
+				dprintf("  %-15s\t%8X%08X  %s\n", 
 					debug_opts[i].name,
 				       (int)(debug_opts[i].mask>>32), 
 				       (int)(debug_opts[i].mask),
@@ -149,37 +196,36 @@ void dbgSetMaskStr(char *mask_str)
 			}
 			else
 			{
-				printf("  %-15s\t        %8X  %s\n", 
+				dprintf("  %-15s\t        %8X  %s\n", 
 					debug_opts[i].name,
 				       (int)(debug_opts[i].mask), 
 				       debug_opts[i].desc);
 			}
-		
-		printf("Exiting.\n");
-		exit(0);
+		return 0;
 	}
 	
 
 	while (opt) {
-		if (!strcmp(opt, "buffer"))
+		if (!strcasecmp(opt, "buffer"))
 			debugToBuffer(16 * 1024);
-		else if (!strcmp(opt, "dump"))
+		else if (!strcasecmp(opt, "dump"))
 			atexit(printDebugBuffer);
 		else
 		{
 			for (i = 0; i < NELEMS(debug_opts); i++)
-				/* probably should strcasecmp() or stricmp() */
-				if (!strcmp(opt, debug_opts[i].name))
+				if (!strcasecmp(opt, debug_opts[i].name))
 				{
 					kaffevmDebugMask |= debug_opts[i].mask;
 					break;
 				}
 		
 			/* Be polite. */
-			if (i == (sizeof debug_opts)/(sizeof(debug_opts[0])))
-				fprintf(stderr, 
+			if (i == (sizeof debug_opts)/(sizeof(debug_opts[0]))){
+				dprintf(
 				    "Unknown flag (%s) passed to -vmdebug\n",
 					opt);
+				return 0;
+			}
 		}
 		
 		/* Get next opt */
@@ -195,6 +241,7 @@ void dbgSetMaskStr(char *mask_str)
 			"You cannot debug the JIT in interpreter mode \n");
 #endif
 	}
+	return 1;
 }
 
 static char *debugBuffer;
@@ -219,8 +266,10 @@ debugToBuffer(int size)
 static void
 debugExitHook(void)
 {
-	// this is a hook for catching exits from GDB.
-	// make this dependent on the selection of this option
+	/*
+	 * this is a hook for catching exits from GDB.
+	 * make this dependent on the selection of this option
+	 */	
 	DBG(BREAKONEXIT, DBGGDBBREAK())
 }
 
@@ -230,10 +279,16 @@ debugExitHook(void)
 static void
 debugSysInit(void)
 {
+	static int once = 0;
+
+	if (once) return;
+	once = 1;
 #if defined(TRANSLATOR)
-	extern int jit_debug;
-	if (getenv("JIT_DEBUG")) 
-		jit_debug = 1;
+	{
+		extern int jit_debug;
+		if (getenv("JIT_DEBUG")) 
+			jit_debug = 1;
+	}
 #endif
 	atexit(debugExitHook);
 }
@@ -261,7 +316,7 @@ printDebugBuffer(void)
 }
 
 int
-dprintf(const char *fmt, ...)
+kaffe_dprintf(const char *fmt, ...)
 {
 
 	int n;
@@ -289,9 +344,17 @@ dprintf(const char *fmt, ...)
 	}
 	else
 	{
+#if 0
+		intsDisable();
+		jthreadRestoreFD(2);
+#endif
 		/* Boring. Print to stderr */
 		n = vfprintf(stderr, fmt, args);
 		fflush(stderr);
+#if 0
+		jthreadedFileDescriptor(2);
+		intsRestore();
+#endif
 	}
 
 	va_end(args);
