@@ -1,3 +1,12 @@
+/*
+ * Copyright (c) 1998, 1999, 2000 The University of Utah. All rights reserved.
+ *
+ * See the file "license.terms" for information on usage and redistribution
+ * of this file.
+ *
+ * Contributed by the Flux Research Group at the University of Utah.
+ * Authors: Godmar Back, Patrick Tullmann 
+ */
 /*
  * debug.h
  *
@@ -8,15 +17,17 @@
  * or, if it's enabled, specific debug statements can be dynamically
  * enabled at run time by setting the vmdebugmask, which is done
  * with the -vmdebug command line switch.
- *
- * INSERT COPYRIGHT HERE
- *
- * Written by Patrick Tullmann <tullmann@cs.utah.edu>, 1998
- *	   and Godmar Back <gback@cs.utah.edu>
  */
 #ifndef __kaffevm_debug_h
 #define __kaffevm_debug_h
 
+/* Pascal Bourguignon <pjb@imaginet.fr> writes:
+ * We include stdio here because on Linux, stdio defines dprintf.
+ * Hence, we can override it with a macro defined here. (stdio.h
+ * is include-once).
+ */
+#include <stdio.h>
+
 /*
  * Available debugging options.
  * For an explanation, refer to the table in debug.c
@@ -25,19 +36,19 @@
  * then #include "debug.h" in every file where you want to use the
  * option, and then use either DBG or DBGEXPR!
  *
- * If you add option, try to invent sensible categories.
+ * If you add options, try to invent sensible categories.
  */
 /* Debug Masks: (1 bit per option) */
 # define DBG_BIT(x) (((jlong)1)<<x)
 # define DBG_NONE		(0)
-# define DBG_VMLOCKS            DBG_BIT(0)
+# define DBG_SLOWLOCKS          DBG_BIT(0)
 # define DBG_VMCONDS		DBG_BIT(1)
 # define DBG_NEWINSTR		DBG_BIT(2)
 # define DBG_VMTHREAD		DBG_BIT(3) 
 # define DBG_EXCEPTION		DBG_BIT(4) 
 # define DBG_JTHREAD		DBG_BIT(5) 
 # define DBG_JTHREADDETAIL	DBG_BIT(6) 
-# define DBG_DETECTDEADLOCK	DBG_BIT(7) 
+# define DBG_STACKTRACE		DBG_BIT(7)
 # define DBG_BREAKONEXIT	DBG_BIT(8) 
 # define DBG_INIT		DBG_BIT(9) 
 # define DBG_GCPRIM		DBG_BIT(10) 
@@ -61,9 +72,39 @@
 # define DBG_NOGC		DBG_BIT(28) 
 # define DBG_JIT		DBG_BIT(29) 
 # define DBG_MOREJIT		DBG_BIT(30) 
+# define DBG_STATICINIT		DBG_BIT(31) 
+# define DBG_RESERROR		DBG_BIT(32)
+# define DBG_GCPRECISE		DBG_BIT(33)
+# define DBG_CLASSFILE		DBG_BIT(34)
+# define DBG_VMCLASSLOADER	DBG_BIT(35)
+# define DBG_GCWALK		DBG_BIT(36)
+# define DBG_GCDIAG		DBG_BIT(37)
+# define DBG_LOCKCONTENTION	DBG_BIT(38)
+# define DBG_CODEANALYSE	DBG_BIT(39)
+# define DBG_CLASSGC		DBG_BIT(40)
+# define DBG_NEWOBJECT		DBG_BIT(41)
+# define DBG_FINALIZE		DBG_BIT(42)
+# define DBG_NATIVELIB		DBG_BIT(43)
+# define DBG_NATIVENET		DBG_BIT(44)
 
-/* let's reserve 50-63 for temporary uses */
-# define DBG_SLACKANAL		DBG_BIT(50) 
+# define DBG_AWT_MEM		DBG_BIT(45)
+# define DBG_AWT_CLR		DBG_BIT(46)
+# define DBG_AWT_EVT		DBG_BIT(47)
+# define DBG_AWT_IMG		DBG_BIT(48)
+# define DBG_AWT_WND		DBG_BIT(49)
+# define DBG_AWT_GRA		DBG_BIT(50)
+# define DBG_AWT_FNT		DBG_BIT(51)
+# define DBG_AWT		(DBG_AWT_MEM | DBG_AWT_CLR | DBG_AWT_EVT    \
+				 | DBG_AWT_IMG | DBG_AWT_WND | DBG_AWT_GRA  \
+				 | DBG_AWT_FNT)
+
+# define DBG_SLACKANAL		DBG_BIT(52) 
+# define DBG_GCJ		DBG_BIT(53) 
+# define DBG_GCJMORE		DBG_BIT(54) 
+# define DBG_INITCLASSPATH      DBG_BIT(55)
+# define DBG_CLASSLOOKUP        DBG_BIT(56)
+
+# define DBG_REGFORCE		DBG_BIT(57)
 
 # define DBG_ALL		((jlong)(-1))
 # define DBG_ANY                DBG_ALL
@@ -94,7 +135,7 @@
  *		to the default value.
  *
  *	Example:  if (DBGEXPR(EXCEPTION, false, true))
- *			catchSignal(SIGSEGV);
+ *			restoreSynchSignalHandler(SIGSEGV);
  *
  *	does not catch SIGSEGV if you're debugging exceptions, but in all
  *	other cases.
@@ -111,20 +152,8 @@
 #elif !defined(KAFFEH) 
 /* --- Debugging is enabled --- */
 
-#include "jtypes.h"
-
 /* Defines what debugging output is seen. Needs to be 64-bit. */
-extern jlong kaffevmDebugMask;
-
-/* Set the debugging mask to use. (give the mask) */
-void dbgSetMask(jlong mask);
-
-/* 
- * Set the debugging mask to use. (give a string, useful for
- * parsing a command line option.  The string is a comma-separated
- * list of options which combined form the debugging mask.
- */
-void dbgSetMaskStr(char *mask_str);
+extern long long kaffevmDebugMask;
 
 # define DBGIF(statement)  statement
 
@@ -142,16 +171,46 @@ void dbgSetMaskStr(char *mask_str);
 /* Do something that would cause GDB to gain control. */
 # define DBGGDBBREAK() { (*(int*)0) = 42; }
 
+/* XXX: change dprintf to kaffe_dprintf and get rid of the macro */
+# define dprintf       kaffe_dprintf
+
+#ifdef __cplusplus
+extern "C"
+#else
+extern
+#endif
+int kaffe_dprintf(const char *fmt, ...);
+
 #else	/* !defined(KAFFEH) */
 
 /* --- give some simple macros for debugging kaffeh */
 
-# define DBG(mask, statement)		statement
-# define DBGEXPR(mask, expr, default) 	expr
-# define DBGIF(statement)  statement
+/* You must define DEBUG_KAFFEH to debug kaffeh */
+# if defined(DEBUG_KAFFEH)
+#  define DBG(mask, statement)		statement
+#  define DBGEXPR(mask, expr, default) 	expr
+#  define DBGIF(statement)  statement
+# else
+#  define DBG(mask, statement)
+#  define DBGEXPR(mask, expr, default) 	default
+#  define DBGIF(statement)
+# endif /* defined(DEBUG_KAFFEH) */
+
 /* we don't link with debug.o, so make dprintf a simple printf */
 # define dprintf	printf
 
 #endif /* defined(NDEBUG) || !defined(DEBUG) */
 
+/* Set the debugging mask to use. (give the mask) */
+void dbgSetMask(long long mask);
+
+/* 
+ * Set the debugging mask to use. (give a string, useful for
+ * parsing a command line option.  The string is a comma-separated
+ * list of options which combined form the debugging mask.
+ *
+ * Return false if nothing was set
+ */
+int dbgSetMaskStr(char *mask_str);
+
 #endif /* __kaffevm_debug_h */
