@@ -17,7 +17,13 @@
 #define	sub_long(t, f1, f2)			(t)[0].v.tlong = (f1)[0].v.tlong - (f2)[0].v.tlong
 #define	mul_long(t, f1, f2)			(t)[0].v.tlong = (f1)[0].v.tlong * (f2)[0].v.tlong
 #define	div_long(t, f1, f2)			(t)[0].v.tlong = (f1)[0].v.tlong / (f2)[0].v.tlong
-#define	rem_long(t, f1, f2)			(t)[0].v.tlong = (f1)[0].v.tlong % (f2)[0].v.tlong
+#if LONG_LONG_MODULO_BROKEN
+/* egcs 1.1.* on IRIX 6.3/mips produces an arithmetic exception for
+   LONG_LONG_MIN % -1ll.  Since anything % -1ll is 0, special-case it.  */
+# define rem_long(t, f1, f2)			(t)[0].v.tlong = (((f2)[0].v.tlong != -1) ? ((f1)[0].v.tlong % (f2)[0].v.tlong) : 0)
+#else
+# define rem_long(t, f1, f2)			(t)[0].v.tlong = (f1)[0].v.tlong % (f2)[0].v.tlong
+#endif
 #define	neg_long(t, f)				(t)[0].v.tlong = -(f)[0].v.tlong
 
 #define	and_long(t, f1, f2)			(t)[0].v.tlong = (f1)[0].v.tlong & (f2)[0].v.tlong
@@ -89,7 +95,7 @@
 #define	xor_int(t, f1, f2)			(t)[0].v.tint = ((f1)[0].v.tint) ^ ((f2)[0].v.tint)
 
 #define	cvt_int_byte(t, f)			(t)[0].v.tint = (((f)[0].v.tint) << 24) >> 24
-#define	cvt_int_char(t, f)			(t)[0].v.tint = ((f)[0].v.tint) & 0xFF
+#define	cvt_int_char(t, f)			(t)[0].v.tint = ((f)[0].v.tint) & 0xFFFF
 #define	cvt_int_short(t, f)			(t)[0].v.tint = (((f)[0].v.tint) << 16) >> 16
 
 #define	branch_indirect(w)			w
@@ -250,8 +256,8 @@
 #define	mul_double(t, f1, f2)			(t)[0].v.tdouble = (f1)[0].v.tdouble * (f2)[0].v.tdouble
 #define	div_float(t, f1, f2)			(t)[0].v.tfloat = (f1)[0].v.tfloat / (f2)[0].v.tfloat
 #define	div_double(t, f1, f2)			(t)[0].v.tdouble = (f1)[0].v.tdouble / (f2)[0].v.tdouble
-#define	rem_float(t, f1, f2)			(t)[0].v.tfloat = remainderf((f1)[0].v.tfloat, (f2)[0].v.tfloat)
-#define	rem_double(t, f1, f2)			(t)[0].v.tdouble = remainder((f1)[0].v.tdouble, (f2)[0].v.tdouble)
+#define	rem_float(t, f1, f2)			(t)[0].v.tfloat = javaRemainderf((f1)[0].v.tfloat, (f2)[0].v.tfloat)
+#define	rem_double(t, f1, f2)			(t)[0].v.tdouble = javaRemainder((f1)[0].v.tdouble, (f2)[0].v.tdouble)
 #define	neg_float(t, f)				(t)[0].v.tfloat = -(f)[0].v.tfloat
 #define	neg_double(t, f)			(t)[0].v.tdouble = -(f)[0].v.tdouble
 
@@ -267,7 +273,7 @@
 #define	cvt_double_int(t, f)			(t)[0].v.tint = floor((f)[0].v.tdouble)
 #define	cvt_double_float(t, f)			(t)[0].v.tfloat = (f)[0].v.tdouble
 
-#define	softcall_lookupmethod(r, n, t)		(r)[0].v.taddr = soft_lookupmethod((t)[0].v.taddr, (n)->name, (n)->signature)
+#define	softcall_lookupinterfacemethod(r, n, t)	(r)[0].v.taddr = soft_lookupinterfacemethod((t)[0].v.taddr, (n)->class, (n)->idx)
 #define	softcall_new(r, t)			(r)->v.taddr = soft_new(t)
 #define	softcall_newarray(r, s, t)		(r)->v.taddr = soft_newarray(t, (s)->v.tint)
 #define	softcall_anewarray(r, s, t)		(r)->v.taddr = soft_anewarray(t, (s)->v.tint)
@@ -276,8 +282,11 @@
 #define softcall_initialise_class(cl)		soft_initialise_class(cl)
 #define	softcall_instanceof(r, o, t)		(r)->v.tint = soft_instanceof(t, (o)->v.taddr)
 
-#define	softcall_monitorenter(o)		lockMutex((o)[0].v.taddr)
-#define	softcall_monitorexit(o)			unlockMutex((o)[0].v.taddr)
+/* Thread.stop() knows how to back out of java frames, so we do not
+ * want to disable stop here
+ */
+#define	softcall_monitorenter(o)		lockObject((o)[0].v.taddr)
+#define	softcall_monitorexit(o)			unlockObject((o)[0].v.taddr)
 
 #define	softcall_multianewarray(r, z, s, t)	(r)->v.taddr = soft_multianewarray(t, z, s)
 
@@ -291,7 +300,7 @@
 
 #define	adjustpc(a)				/* Not needed for interpreter */
 
-#define	check_array_index(O, I)			if ((I)[0].v.tint >= ((struct Array*)((O)[0].v.taddr))->length) { \
+#define	check_array_index(O, I)			if ((I)[0].v.tint >= ARRAY_SIZE((O)[0].v.taddr)) { \
 							soft_badarrayindex(); \
 						}
 #define	build_call_frame(SIG, OBJ, NRARGS)	/* Not needed for interpreter */
