@@ -226,7 +226,7 @@ void check_array_index(SlotInfo*, SlotInfo*);
 void check_array_constindex(SlotInfo*, jint);
 
 
-void softcall_lookupmethod(SlotInfo*, Method*, SlotInfo*);
+void softcall_lookupinterfacemethod(SlotInfo*, Method*, SlotInfo*);
 void softcall_nullpointer(void);
 void softcall_new(SlotInfo*, struct Hjava_lang_Class*);
 void softcall_newarray(SlotInfo*, SlotInfo*, int);
