@@ -25,6 +25,8 @@
 #include "constants.h"
 #include "classMethod.h"
 #include "gc.h"
+#include "support.h"
+#include "stats.h"
 #include "itypes.h"
 #include "md.h"
 #include "locks.h"
@@ -48,15 +50,6 @@ void fixupFunctionCall(sequence*);
 void syncRegisters(sequence*);
 void nowritebackSlot(sequence*);
 
-extern uint32 pc;
-extern uint32 npc;
-extern int maxPush;
-extern int maxArgs;
-extern int maxTemp;
-extern int maxLocal;
-extern int maxStack;
-extern int isStatic;
-
 bool used_ieee_rounding;
 bool used_ieee_division;
 
@@ -251,7 +244,7 @@ mon_enter(methods* meth, SlotInfo* obj)
 	else {
 		pusharg_ref(obj, 0);
 	}
-	call_soft(lockMutex);
+	call_soft(lockObject);
 	popargs();
 	start_sub_block();
 }
@@ -266,7 +259,7 @@ mon_exit(methods* meth, SlotInfo* obj)
 	else {
 		pusharg_ref(obj, 0);
 	}
-	call_soft(unlockMutex);
+	call_soft(unlockObject);
 	popargs();
 	start_sub_block();
 }
@@ -1015,34 +1008,26 @@ void
 rem_float(SlotInfo* dst, SlotInfo* src, SlotInfo* src2)
 {
 	used_ieee_division = true;
-#if defined(HAVE_rem_float)
-	slot_slot_slot(dst, src, src2, HAVE_rem_float, Tcomplex);
-#else
 	end_sub_block();
 	pusharg_float(src2, 1);
 	pusharg_float(src, 0);
 	call_soft(soft_frem);
 	popargs();
-	end_sub_block();
+	start_sub_block();
 	return_float(dst);
-#endif
 }
 
 void
 rem_double(SlotInfo* dst, SlotInfo* src, SlotInfo* src2)
 {
 	used_ieee_division = true;
-#if defined(HAVE_rem_double)
-	lslot_lslot_lslot(dst, src, src2, HAVE_rem_double, Tcomplex);
-#else
 	end_sub_block();
 	pusharg_double(src2, pusharg_long_idx_inc);
 	pusharg_double(src, 0);
 	call_soft(soft_freml);
 	popargs();
-	end_sub_block();
+	start_sub_block();
 	return_double(dst);
-#endif
 }
 
 void
@@ -1095,7 +1080,7 @@ void
 neg_float(SlotInfo* dst, SlotInfo* src)
 {
 #if defined(HAVE_neg_float)
-	lslot_lslot_lslot(dst, 0, src, HAVE_neg_float, Tcomplex);
+	slot_slot_slot(dst, 0, src, HAVE_neg_float, Tcomplex);
 #else
 	SlotInfo* zero;
 	slot_alloctmp(zero);
@@ -1470,6 +1455,7 @@ load_offset_ref(SlotInfo* dst, SlotInfo* src, jint offset)
 		slot_alloctmp(tmp);
 		add_ref_const(tmp, src, offset);
 		load_ref(dst, tmp);
+		slot_freetmp(tmp);
 	}
 }
 
@@ -1496,6 +1482,7 @@ load_long(SlotInfo* dst, SlotInfo* src)
 	/* Don't use LSLOT & HSLOT here */
 	load_int(dst, src);
 	load_int(dst+1, tmp);
+	slot_freetmp(tmp);
 #endif
 }
 
@@ -1503,7 +1490,7 @@ void
 load_addr_long(SlotInfo* dst, void* addr)
 {
 	load_addr_int(dst, addr);
-	load_addr_int(dst+1, addr+4);
+	load_addr_int(dst+1, (char*)addr+4);
 }
 
 #if defined(HAVE_load_float)
@@ -1644,6 +1631,7 @@ load_offset_byte(SlotInfo* dst, SlotInfo* src, jint offset)
 		slot_alloctmp(tmp);
 		add_ref_const(tmp, src, offset);
 		load_byte(dst, tmp);
+		slot_freetmp(tmp);
 	}
 }
 
@@ -1665,6 +1653,7 @@ load_offset_char(SlotInfo* dst, SlotInfo* src, jint offset)
 		slot_alloctmp(tmp);
 		add_ref_const(tmp, src, offset);
 		load_char(dst, tmp);
+		slot_freetmp(tmp);
 	}
 }
 
@@ -1686,6 +1675,7 @@ load_offset_short(SlotInfo* dst, SlotInfo* src, jint offset)
 		slot_alloctmp(tmp);
 		add_ref_const(tmp, src, offset);
 		load_short(dst, tmp);
+		slot_freetmp(tmp);
 	}
 }
 
@@ -1707,6 +1697,7 @@ load_offset_float(SlotInfo* dst, SlotInfo* src, jint offset)
 		slot_alloctmp(tmp);
 		add_ref_const(tmp, src, offset);
 		load_float(dst, tmp);
+		slot_freetmp(tmp);
 	}
 }
 
@@ -1728,6 +1719,7 @@ load_offset_double(SlotInfo* dst, SlotInfo* src, jint offset)
 		slot_alloctmp(tmp);
 		add_ref_const(tmp, src, offset);
 		load_double(dst, tmp);
+		slot_freetmp(tmp);
 	}
 }
 
@@ -1866,6 +1858,7 @@ store_offset_int(SlotInfo* dst, jint offset, SlotInfo* src)
 		slot_alloctmp(tmp);
 		add_ref_const(tmp, dst, offset);
 		store_int(tmp, src);
+		slot_freetmp(tmp);
 	}
 }
 
@@ -1905,6 +1898,7 @@ store_offset_ref(SlotInfo* dst, jint offset, SlotInfo* src)
 		slot_alloctmp(tmp);
 		add_ref_const(tmp, dst, offset);
 		store_ref(tmp, src);
+		slot_freetmp(tmp);
 	}
 }
 
@@ -1990,6 +1984,7 @@ store_offset_float(SlotInfo* dst, jint offset, SlotInfo* src)
 		slot_alloctmp(tmp);
 		add_ref_const(tmp, dst, offset);
 		store_float(tmp, src);
+		slot_freetmp(tmp);
 	}
 }
 
@@ -2029,6 +2024,7 @@ store_offset_double(SlotInfo* dst, jint offset, SlotInfo* src)
 		slot_alloctmp(tmp);
 		add_ref_const(tmp, dst, offset);
 		store_double(tmp, src);
+		slot_freetmp(tmp);
 	}
 }
 
@@ -2059,6 +2055,8 @@ store_byte(SlotInfo* dst, SlotInfo* src)
 	and_int_const(tmp2, tmp2, -(1 << (8 * sizeof(jbyte))));
 	or_int(tmp2, tmp2, tmp);
 	store_int(dst, tmp2);
+	slot_freetmp(tmp2);
+	slot_freetmp(tmp);
 #endif
 }
 
@@ -2080,6 +2078,7 @@ store_offset_byte(SlotInfo* dst, jint offset, SlotInfo* src)
 		slot_alloctmp(tmp);
 		add_ref_const(tmp, dst, offset);
 		store_byte(tmp, src);
+		slot_freetmp(tmp);
 	}
 }
 
@@ -2110,6 +2109,8 @@ store_char(SlotInfo* dst, SlotInfo* src)
 	and_int_const(tmp2, tmp2, -(1 << (8 * sizeof(jchar))));
 	or_int(tmp2, tmp2, tmp);
 	store_int(dst, tmp2);
+	slot_freetmp(tmp2);
+	slot_freetmp(tmp);
 #endif
 }
 
@@ -2131,6 +2132,7 @@ store_offset_char(SlotInfo* dst, jint offset, SlotInfo* src)
 		slot_alloctmp(tmp);
 		add_ref_const(tmp, dst, offset);
 		store_char(tmp, src);
+		slot_freetmp(tmp);
 	}
 }
 
@@ -2161,6 +2163,8 @@ store_short(SlotInfo* dst, SlotInfo* src)
 	and_int_const(tmp2, tmp2, -(1 << (8 * sizeof(jshort))));
 	or_int(tmp2, tmp2, tmp);
 	store_int(dst, tmp2);
+	slot_freetmp(tmp2);
+	slot_freetmp(tmp);
 #endif
 }
 
@@ -2182,6 +2186,7 @@ store_offset_short(SlotInfo* dst, jint offset, SlotInfo* src)
 		slot_alloctmp(tmp);
 		add_ref_const(tmp, dst, offset);
 		store_short(tmp, src);
+		slot_freetmp(tmp);
 	}
 }
 
@@ -2298,6 +2303,8 @@ store_const_offset_int(SlotInfo* dst, jint offset, jint val)
 		add_ref_const(tmp, dst, offset);
 		move_int_cosnt(tmp2, val);
 		store_int(tmp, tmp2);
+		slot_freetmp(tmp2);
+		slot_freetmp(tmp);
 	}
 }
 #endif
@@ -2318,6 +2325,8 @@ store_const_offset_byte(SlotInfo* dst, jint offset, jint val)
 		add_ref_const(tmp, dst, offset);
 		move_int_cosnt(tmp2, val);
 		store_byte(tmp, tmp2);
+		slot_freetmp(tmp2);
+		slot_freetmp(tmp);
 	}
 }
 #endif
@@ -2342,6 +2351,7 @@ pusharg_int_const(int val, int idx)
 		slot_alloctmp(tmp);
 		move_int_const(tmp, val);
 		pusharg_int(tmp, idx);
+		slot_freetmp(tmp);
 	}
 }
 
@@ -2378,6 +2388,7 @@ pusharg_ref_const(void* val, int idx)
 		slot_alloctmp(tmp);
 		move_ref_const(tmp, val);
 		pusharg_ref(tmp, idx);
+		slot_freetmp(tmp);
 	}
 }
 
@@ -2486,6 +2497,7 @@ cbranch_int_const(SlotInfo* s1, jint val, label* dst, int type)
 		slot_alloctmp(tmp);
 		move_int_const(tmp, val);
 		cbranch_int(s1, tmp, dst, type);
+		slot_freetmp(tmp);
 	}
 #else
 	cmp_int_const(0, s1, val);
@@ -2519,6 +2531,7 @@ cbranch_ref_const(SlotInfo* s1, void *val, label* dst, int type)
 		slot_alloctmp(tmp);
 		move_ref_const(tmp, val);
 		cbranch_ref(s1, tmp, dst, type);
+		slot_freetmp(tmp);
 	}
 #else
 	cmp_ref_const(0, s1, val);
@@ -2549,17 +2562,21 @@ void
 call_indirect_method(Method *meth)
 {
 	void* ptr;
-	SlotInfo *tmp;
 
-	ptr = &METHOD_NATIVECODE(meth);
+	ptr = PMETHOD_NATIVECODE(meth);
 
 #if defined(HAVE_call_indirect_const)
 	slot_const_const(0, (jword)ptr, ba, HAVE_call_indirect_const, Tnull);
 #else
-	slot_alloctmp(tmp);
-	move_ref_const(tmp, ptr);
-	load_ref(tmp, tmp);
-	call(tmp);
+	{
+		SlotInfo *tmp;
+
+		slot_alloctmp(tmp);
+		move_ref_const(tmp, ptr);
+		load_ref(tmp, tmp);
+		call(tmp);
+		slot_freetmp(tmp);
+	}
 #endif
 }
 
@@ -2568,7 +2585,7 @@ call_soft(void *routine)
 {
 #if defined(HAVE_call_soft)
 	label* l = newLabel();
-	l->type = Labsolute|Lexternal;
+	l->type = Labsolute|Lexternal|Lnoprofile;
 	l->at = 0;
 	l->to = (uintp)routine;	/* What place does it goto */
 	l->from = 0;
@@ -2577,7 +2594,7 @@ call_soft(void *routine)
 #elif defined(HAVE_call_ref)
 	label* l;
 	l = newLabel();
-	l->type = Lexternal;
+	l->type = Lexternal|Lnoprofile;
 	l->at = 0;
 	l->to = (uintp)routine;	/* What place does it goto */
 	l->from = 0;
@@ -2599,6 +2616,7 @@ call_soft(void *routine)
 	move_label_const(tmp, l);
 	load_ref(tmp, tmp);
 	call(tmp);
+	slot_freetmp(tmp);
 #endif
 }
 
@@ -2830,6 +2848,7 @@ cmp_int_const(SlotInfo* dst, SlotInfo* src, jint val)
 		slot_alloctmp(tmp);
 		move_int_const(tmp, val);
 		cmp_int(dst, src, tmp);
+		slot_freetmp(tmp);
 	}
 }
 
@@ -2855,6 +2874,7 @@ cmp_ref_const(SlotInfo* dst, SlotInfo* src, void* val)
 		slot_alloctmp(tmp);
 		move_ref_const(tmp, val);
 		cmp_ref(dst, src, tmp);
+		slot_freetmp(tmp);
 	}
 }
 
@@ -3176,12 +3196,15 @@ build_call_frame(Utf8Const* sig, SlotInfo* obj, int sp_idx)
 	static int sz_args = 0;
 	int arg_idx;
 	int idx;
-	char* sigptr;
+	const char* sigptr;
 
 	/* Make sure we have enough argument space */
 	if (sp_idx + 1 > sz_args) {
+		/* add differential */
+		addToCounter(&jitmem, "jitmem-temp", 1,
+			(sp_idx + 1 - sz_args) * sizeof(struct pusharg_info));
 		sz_args = sp_idx + 1;
-		args = gc_realloc_fixed(args, sizeof(struct pusharg_info) * sz_args);
+		args = KREALLOC(args, sizeof(struct pusharg_info) * sz_args);
 	}
 
 	/* If we've got an object ... */
@@ -3280,14 +3303,15 @@ build_call_frame(Utf8Const* sig, SlotInfo* obj, int sp_idx)
 /*									   */
 
 void
-softcall_lookupmethod(SlotInfo* dst, Method* meth, SlotInfo* obj)
+softcall_lookupinterfacemethod(SlotInfo* dst, Method* meth, SlotInfo* obj)
 {
 	/* 'obj' must be written back since it will be reused */
 	prepare_function_call();
-	pusharg_utf8_const(meth->signature, 2);
-	pusharg_utf8_const(meth->name, 1);
+	/* FIXME: should there be a pusharg_int_const? */
+	pusharg_ref_const((void*)(int)meth->idx, 2);		
+	pusharg_class_const(meth->class, 1);
 	pusharg_ref(obj, 0);
-	call_soft(soft_lookupmethod);
+	call_soft(soft_lookupinterfacemethod);
 	popargs();
 	fixup_function_call();
 	return_ref(dst);
@@ -3303,11 +3327,10 @@ check_array_index(SlotInfo* obj, SlotInfo* idx)
 #if defined(HAVE_ccall_ugt)
 	ccall_int_ugt(tmp, idx, soft_badarrayindex);
 #else
+	end_sub_block();
 	cbranch_int_ult(idx, tmp, reference_label(1, 1));
-	if (canCatch(BADARRAYINDEX)) {
-		sync_registers();
-	}
 	call_soft(soft_badarrayindex);
+	start_sub_block();
 	set_label(1, 1);
 #endif
 	slot_freetmp(tmp);
@@ -3323,11 +3346,10 @@ check_array_constindex(SlotInfo* obj, jint idx)
 #if defined(HAVE_ccall_ugt)
 	ccall_int_const_ugt(tmp, idx, soft_badarrayindex);
 #else
+	end_sub_block();
 	cbranch_int_const_ugt(tmp, idx, reference_label(1, 1));
-	if (canCatch(BADARRAYINDEX)) {
-		sync_registers();
-	}
 	call_soft(soft_badarrayindex);
+	start_sub_block();
 	set_label(1, 1);
 #endif
 	slot_freetmp(tmp);
@@ -3479,7 +3501,7 @@ softcall_monitorenter(SlotInfo* mon)
 	slot_nowriteback(mon);
 	prepare_function_call();
 	pusharg_ref(mon, 0);
-	call_soft(lockMutex);
+	call_soft(lockObject);
 	popargs();
 	fixup_function_call();
 }
@@ -3490,7 +3512,7 @@ softcall_monitorexit(SlotInfo* mon)
 	slot_nowriteback(mon);
 	prepare_function_call();
 	pusharg_ref(mon, 0);
-	call_soft(unlockMutex);
+	call_soft(unlockObject);
 	popargs();
 	fixup_function_call();
 }
