@@ -32,7 +32,17 @@
 /* -------------------------------------------------------------------- */
 /* Methods */
 
-#define	get_method_info(idx)  getMethodSignatureClass(idx, meth->class, true, &cinfo)
+#define	get_method_info(idx)  \
+  if (getMethodSignatureClass(idx, meth->class, true, 0, &cinfo, einfo) \
+	== false) { success = false; goto done; }
+
+#define	get_special_method_info(idx)  \
+  if (getMethodSignatureClass(idx, meth->class, true, 1, &cinfo, einfo) \
+	== false) { success = false; goto done; } 
+
+#define	get_interface_method_info(idx)  \
+  if (getMethodSignatureClass(idx, meth->class, true, 2, &cinfo, einfo) \
+	== false) { success = false; goto done; } 
 
 #define	method_name()	(cinfo.name)
 #define	method_sig()	(cinfo.signature)
@@ -50,17 +60,26 @@
 /* Fields */
 
 #define	get_field_info(IDX) \
-				getField((IDX), meth->class, false, &finfo)
+  if (getField((IDX), meth->class, false, &finfo, einfo) == false) { \
+    success = false; goto done; \
+  }
 
 #define	get_static_field_info(IDX) \
-				getField((IDX), meth->class, true, &finfo)
+  if (getField((IDX), meth->class, true, &finfo, einfo) == false) { \
+    success = false; goto done; \
+  }
 
 #define field_class()		(finfo.class)
+#define field_statics()		(finfo.class->static_data)
 
 /* -------------------------------------------------------------------- */
 /* Classes */
 
-#define	get_class_info(IDX)	crinfo = getClass((IDX), meth->class)
+#define	get_class_info(IDX)	\
+  crinfo = getClass((IDX), meth->class, einfo); \
+  if (crinfo == 0) { \
+    success = false; goto done; \
+  }
 
 #define	class_object()		(crinfo)
 
@@ -70,6 +89,15 @@
 #define	object_array_offset	(ARRAY_DATA_OFFSET)
 #define	object_array_length	(ARRAY_SIZE_OFFSET)
 
+/* -------------------------------------------------------------------- */
+/* Errors */
+
+#define compile_time_error(EINFO) {		\
+	*einfo = (EINFO);			\
+	success = false;			\
+	goto done;				\
+}
+
 /* -------------------------------------------------------------------- */
 /* Switches */
 
@@ -87,7 +115,7 @@
 #endif
 
 struct _methods;
-void translate(struct _methods*);
+bool translate(struct _methods*, errorInfo*);
 
 typedef struct _nativeCodeInfo {
 	void*	mem;
@@ -96,6 +124,12 @@ typedef struct _nativeCodeInfo {
 	int	codelen;
 } nativeCodeInfo;
 
+struct codeinfo;
+bool initInsnSequence(Method *meth, int codesize, int localsz, int stacksz,
+		      struct _errorInfo *einfo);
+bool finishInsnSequence(struct codeinfo*, nativeCodeInfo*, errorInfo*);
+void installMethodCode(struct codeinfo*, Method*, nativeCodeInfo*);
+
 typedef struct {
         bool BADARRAYINDEX;
         bool NULLPOINTER;
@@ -103,7 +137,29 @@ typedef struct {
 
 extern jitflags willcatch;
 
-#define	willCatch(FLAG)	willcatch.##FLAG = true
-#define	canCatch(FLAG)	willcatch.##FLAG
+#define	willCatch(FLAG)	willcatch. FLAG = true
+#define	canCatch(FLAG)	willcatch. FLAG
+
+#include "locks.h"
+extern iLock* translatorlock;
+
+#define	enterTranslator()	lockStaticMutex(&translatorlock)
+#define	leaveTranslator()	unlockStaticMutex(&translatorlock)
+
+extern int maxLocal;
+extern int maxStack;
+extern int maxArgs;
+extern int maxTemp;
+extern int maxPush;
+extern int isStatic;
+extern int CODEPC;
+extern nativecode* codeblock;
+extern uint32 pc;
+extern uint32 npc;
+
+#if defined(KAFFE_PROFILER)
+extern int profFlag;
+extern Method *globalMethod;
+#endif
 
 #endif
