@@ -1,5 +1,5 @@
 /*
- * fastlocks.c
+ * locks.c
  * Manage locking system
  *
  * Copyright (c) 1996-1999
@@ -9,8 +9,6 @@
  * of this file. 
  */
 
-#define	LDBG(s)
-
 #include "config.h"
 #include "config-std.h"
 #include "object.h"
@@ -18,15 +16,18 @@
 #include "baseClasses.h"
 #include "thread.h"
 #include "locks.h"
+#include "ksem.h"
 #include "errors.h"
 #include "exception.h"
 #include "md.h"
 #include "jthread.h"
+#include "debug.h"
 
 /*
- * If we don't have an atomic compare and exchange defined then make one
- * out of a simple atmoc exchange (using the LOCKINPROGRESS value the place
- * holder).  If we don't have that, we'll just fake it.
+ * If we don't have an atomic compare and exchange defined then make
+ * one out of a simple atomic exchange (using the LOCKINPROGRESS value
+ * as the place holder).  If we don't have ATOMIC_EXCHANGE, we'll just
+ * fake it.
  */
 #if !defined(COMPARE_AND_EXCHANGE)
 #if defined(ATOMIC_EXCHANGE)
@@ -65,10 +66,7 @@ static struct {
 };
 #define	NR_SPECIAL_LOCKS	(sizeof(specialLocks)/sizeof(specialLocks[0]))
 
-static jboolean _SemGet(void*, jlong);
-static void _SemPut(void*);
-
-/* Count number of backoffs */
+/* Count number of backoffs.  XXX move into the STAT infrastructure. */
 int backoffcount = 0;
 
 /*
@@ -100,24 +98,36 @@ getHeavyLock(iLock** lkp)
 	jlong timeout;
 	int i;
 
+DBG(SLOWLOCKS,
+    	dprintf("  getHeavyLock(**lkp=%p, *lk=%p, th=%p)\n",
+		lkp, *lkp, jthread_current());
+)
+ 
 	timeout = 1;
 	for (;;) {
-		/* Get the current lock and replace it with -1 to indicate
+		/* Get the current lock and replace it with LOCKINPROGRESS to indicate
 		 * changes are in progress.
 		 */
 		old = *lkp;
 		if (old == LOCKINPROGRESS || !COMPARE_AND_EXCHANGE(lkp, old, LOCKINPROGRESS)) {
+			/* Someone else put the lock in LOCKINPROGRESS state */
 			tid = getCurrentThread();
 			backoffcount++;
-			SEMGET(unhand(tid)->sem, timeout);
+			ksemGet((Ksem*)(unhand(tid)->sem), timeout);
 			/* Back off */
 			timeout = (timeout << 1)|timeout;
 			continue;
 		}
+
+		/* If bottom bit is set, strip off and use pointer as pointer to heavy lock */
 		if ((((uintp)old) & 1) == 1) {
+DBG(SLOWLOCKS,
+    			dprintf("    got cached lock\n");
+)
 			lk = (iLock*)(((uintp)old) & (uintp)-2);
 		}
 		else {
+			/* Create a heavy lock object for others to find. */
 			lk = 0;
 			for (i = 0; i < NR_SPECIAL_LOCKS; i++) {
 				if (specialLocks[i].key == lkp) {
@@ -125,6 +135,10 @@ getHeavyLock(iLock** lkp)
 					break;
 				}
 			}
+DBG(SLOWLOCKS,
+    			dprintf("    got %s lock\n",
+				(lk == 0) ? "new" : "special");
+)
 			if (lk == 0) {
 				lk = (iLock*)jmalloc(sizeof(iLock));
 			}
@@ -145,6 +159,12 @@ void
 putHeavyLock(iLock** lkp, iLock* lk)
 {
 	assert(*lkp == LOCKINPROGRESS);
+
+DBG(SLOWLOCKS,
+	dprintf("  putHeavyLock(**lkp=%p, *lk=%p, th=%p)\n", 
+		lkp, lk, jthread_current());
+)
+
 	if (lk == LOCKFREE) {
 		*lkp = LOCKFREE;
 	}
@@ -157,13 +177,17 @@ putHeavyLock(iLock** lkp, iLock* lk)
  * Slowly lock a mutex.  We get the heavy lock and lock that instead.
  * If we can't lock it we suspend until we can.
  */
-void
+static void
 slowLockMutex(iLock** lkp, void* where)
 {
 	iLock* lk;
 	Hjava_lang_Thread* tid;
 
-LDBG(	printf("Slow lock\n");						)
+DBG(SLOWLOCKS,
+    	dprintf("slowLockMutex(**lkp=%p, where=%p, th=%p)\n",
+	       lkp, where, jthread_current());
+)
+	jthread_disable_stop(); /* protect the heavy lock, and its queues */
 
 	for (;;) {
 		lk = getHeavyLock(lkp);
@@ -171,6 +195,7 @@ LDBG(	printf("Slow lock\n");						)
 		/* If I hold the heavy lock then just keep on going */
 		if (jthread_on_current_stack(lk->holder)) {
 			putHeavyLock(lkp, lk);
+			jthread_enable_stop();
 			return;
 		}
 
@@ -178,6 +203,7 @@ LDBG(	printf("Slow lock\n");						)
 		if (lk->holder == 0) {
 			lk->holder = where;
 			putHeavyLock(lkp, lk);
+			jthread_enable_stop();
 			return;
 		}
 
@@ -186,7 +212,7 @@ LDBG(	printf("Slow lock\n");						)
 		unhand(tid)->nextlk = lk->mux;
 		lk->mux = tid;
 		putHeavyLock(lkp, lk);
-		SEMGET(unhand(tid)->sem, 0);
+		ksemGet((Ksem*)(unhand(tid)->sem), 0);
 	}
 }
 
@@ -195,26 +221,31 @@ LDBG(	printf("Slow lock\n");						)
  * so they can claim the lock.  If no one is waiting we revert the lock to
  * a fast thin lock.
  */
-void
+static void
 slowUnlockMutex(iLock** lkp, void* where)
 {
 	iLock* lk;
 	Hjava_lang_Thread* tid;
 	int i;
 
-LDBG(	printf("Slow unlock\n");					)
-
+DBG(SLOWLOCKS,
+    	dprintf("slowUnlockMutex(**lkp=%p, where=%p, th=%p)\n",
+	       lkp, where, jthread_current());
+)
+	jthread_disable_stop(); /* protect the heavy lock, and its queues */
 	lk = getHeavyLock(lkp);
 
 	/* Only the lock holder can be doing an unlock */
 	if (!jthread_on_current_stack(lk->holder)) {
 		putHeavyLock(lkp, lk);
+		jthread_enable_stop();
 		throwException(IllegalMonitorStateException);
 	}
 
 	/* If holder isn't where we are now then this isn't the final unlock */
 	if (lk->holder > where) {
 		putHeavyLock(lkp, lk);
+		jthread_enable_stop();
 		return;
 	}
 
@@ -227,7 +258,7 @@ LDBG(	printf("Slow unlock\n");					)
 		unhand(tid)->nextlk = 0;
 		lk->holder = 0;
 		putHeavyLock(lkp, lk);
-		SEMPUT(unhand(tid)->sem);
+		ksemPut((Ksem*)(unhand(tid)->sem));
 	}
 	/* If someone's waiting to be signaled keep the heavy in place */
 	else if (lk->cv != 0) {
@@ -246,6 +277,7 @@ LDBG(	printf("Slow unlock\n");					)
 		}
 		putHeavyLock(lkp, LOCKFREE);
 	}
+	jthread_enable_stop();
 }
 
 void
@@ -254,6 +286,11 @@ _slowUnlockMutexIfHeld(iLock** lkp, void* where)
 	iLock* lk;
 	void* holder;
 
+DBG(SLOWLOCKS,
+    	dprintf("slowUnlockMutexIfHeld(**lkp=%p, where=%p, th=%p)\n",
+	       lkp, where, jthread_current());
+)
+
 	lk = getHeavyLock(lkp);
 	holder = lk->holder;
 	putHeavyLock(lkp, lk);
@@ -269,6 +306,11 @@ _releaseLock(iLock** lkp)
 	iLock* lk;
 	void* holder;
 
+DBG(SLOWLOCKS,
+    	dprintf("_releaseLock(**lkp=%p, th=%p)\n",
+	       lkp, jthread_current());
+)
+
 	lk = getHeavyLock(lkp);
 	holder = lk->holder;
 
@@ -296,6 +338,11 @@ _waitCond(iLock** lkp, jlong timeout)
 	Hjava_lang_Thread** ptr;
 	jboolean r;
 
+DBG(SLOWLOCKS,
+    	dprintf("_waitCond(**lkp=%p, timeout=%ld, th=%p)\n",
+	       lkp, (long)timeout, jthread_current());
+)
+
 	lk = getHeavyLock(lkp);
 	holder = lk->holder;
 
@@ -310,7 +357,7 @@ _waitCond(iLock** lkp, jlong timeout)
 	lk->cv = tid;
 	putHeavyLock(lkp, lk);
 	slowUnlockMutex(lkp, holder);
-	r = SEMGET(unhand(tid)->sem, timeout);
+	r = ksemGet((Ksem*)unhand(tid)->sem, timeout);
 
 	/* Timeout */
 	if (r == false) {
@@ -333,7 +380,7 @@ _waitCond(iLock** lkp, jlong timeout)
 		/* Not on list - so must have been signalled after all -
 		 * decrease the semaphore to avoid problems.
 		 */
-		SEMGET(unhand(tid)->sem, 0);
+		ksemGet((Ksem*)(unhand(tid)->sem), 0);
 
 		found:;
 		putHeavyLock(lkp, lk);
@@ -350,6 +397,11 @@ _signalCond(iLock** lkp)
 	iLock* lk;
 	Hjava_lang_Thread* tid;
 
+DBG(SLOWLOCKS,
+    	dprintf("_signalCond(**lkp=%p, th=%p)\n",
+	       lkp, jthread_current());
+)
+
 	lk = getHeavyLock(lkp);
 
 	if (!jthread_on_current_stack(lk->holder)) {
@@ -374,6 +426,11 @@ _broadcastCond(iLock** lkp)
 	iLock* lk;
 	Hjava_lang_Thread* tid;
 
+DBG(SLOWLOCKS,
+    	dprintf("_broadcastCond(**lkp=%p, th=%p)\n",
+	       lkp, jthread_current());
+)
+
 	lk = getHeavyLock(lkp);
 
 	if (!jthread_on_current_stack(lk->holder)) {
@@ -410,6 +467,7 @@ _lockMutex(iLock** lkp, void* where)
 		}
 	}
 	else if (val - (uintp)where > 1024) {
+		/* XXX count this in the stats area */
 		slowLockMutex(lkp, where);
 	}
 }
@@ -428,8 +486,8 @@ _unlockMutex(iLock** lkp, void* where)
 	if ((val & 1) != 0) {
 		slowUnlockMutex(lkp, where);
 	}
-	else if (val == (uintp)where &&
-			!COMPARE_AND_EXCHANGE(lkp, (iLock*)where, LOCKFREE)) {
+	else if ((val == (uintp)where) /* XXX squirrely bit */
+		&& !COMPARE_AND_EXCHANGE(lkp, (iLock*)where, LOCKFREE)) {
 		slowUnlockMutex(lkp, where);
 	}
 }
@@ -463,52 +521,3 @@ dumpLocks(void)
 {
 }
 
-/******************************************************************************
- *
- *  The following routines are used to convert POSIX style locks into the
- *  semaphores we actually need here.
- *
- ******************************************************************************/
-
-#if defined(SETUP_POSIX_LOCKS)
-static
-jboolean
-_SemGet(void* sem, jlong timeout)
-{
-	jboolean r;
-	sem2posixLock* lk;
-
-	r = true;
-	lk = (sem2posixLock*)sem;
-
-	LOCK(lk);
-	if (lk->count == 0) {
-		(void)WAIT(lk, timeout);
-	}
-	if (lk->count == 1) {
-		lk->count = 0;
-		r = true;
-	}
-	else {
-		assert(lk->count == 0);
-		r = false;
-	}
-	UNLOCK(lk);
-	return (r);
-}
-
-static
-void
-_SemPut(void* sem)
-{
-	sem2posixLock* lk;
-
-	lk = (sem2posixLock*)sem;
-
-	LOCK(lk);
-	assert(lk->count == 0);
-        lk->count = 1;
-	SIGNAL(lk);
-	UNLOCK(lk);
-}
-#endif
