@@ -24,6 +24,7 @@
 #include "errors.h"
 #include "md.h"
 #include "stats.h"
+#include "classMethod.h"
 
 /* Avoid recursively allocating OutOfMemoryError */
 #define OOM_ALLOCATING		((void *) -1)
@@ -186,6 +187,38 @@ gc_heap_isobject(gc_block *info, gc_unit *unit)
 	return 0;
 }
 
+static inline void
+markObjectDontCheck(gc_unit *unit, gc_block *info, int idx)
+{
+	/* If object's been traced before, don't do it again */
+	if (GC_GET_COLOUR(info, idx) != GC_COLOUR_WHITE) {
+		return;
+	}
+DBG(GCWALK,	
+	dprintf("  marking @%08p: %s\n", UTOMEM(unit),
+			describeObject(UTOMEM(unit)));
+    )
+
+	DBG(GCSTAT,
+	    switch (GC_GET_FUNCS(info, idx)) {
+	    case GC_ALLOC_NORMALOBJECT:
+	    case GC_ALLOC_FINALIZEOBJECT:
+	    case GC_ALLOC_PRIMARRAY:
+	    case GC_ALLOC_REFARRAY: {
+		    Hjava_lang_Class *c
+			    = OBJECT_CLASS((Hjava_lang_Object *) (unit+1));
+		    c->live_count++;
+	    }});
+	    
+
+	/* If we found a new white object, mark it as grey and
+	 * move it into the grey list.
+	 */
+	GC_SET_COLOUR(info, idx, GC_COLOUR_GREY);
+	UREMOVELIST(unit);
+	UAPPENDLIST(gclists[grey], unit);
+}
+
 /*
  * Mark the memory given by an address if it really is an object.
  */
@@ -208,40 +241,17 @@ gcMarkAddress(Collector* gcif, const void* mem)
 	}
 }
 
-static void
-markObjectDontCheck(gc_unit *unit, gc_block *info, int idx)
-{
-	/* If object's been traced before, don't do it again */
-	if (GC_GET_COLOUR(info, idx) != GC_COLOUR_WHITE) {
-		return;
-	}
-DBG(GCWALK,	
-	dprintf("  marking @%08p: %s\n", UTOMEM(unit),
-			describeObject(UTOMEM(unit)));
-    )
-
-	/* If we found a new white object, mark it as grey and
-	 * move it into the grey list.
-	 */
-	GC_SET_COLOUR(info, idx, GC_COLOUR_GREY);
-	UREMOVELIST(unit);
-	UAPPENDLIST(gclists[grey], unit);
-}
-
 /*
- * Mark an object.  Argument is assumed to point to a valid object.
+ * Mark an object.  Argument is assumed to point to a valid object,
+ * and never, ever, be null.
  */
 static void
 gcMarkObject(Collector* gcif, const void* objp)
 {
-	if (objp != 0) {
-		gc_unit *unit = UTOUNIT(objp);
-		gc_block *info = GCMEM2BLOCK(unit);
-#if defined(DEBUG)
-		assert(gc_heap_isobject(info, unit));
-#endif
-		markObjectDontCheck(unit, info, GCMEM2IDX(info, unit));
-	}
+  gc_unit *unit = UTOUNIT(objp);
+  gc_block *info = GCMEM2BLOCK(unit);
+  DBG(GCDIAG, assert(gc_heap_isobject(info, unit)));
+  markObjectDontCheck(unit, info, GCMEM2IDX(info, unit));
 }
 
 static void
@@ -378,12 +388,29 @@ DBG(GCWALK,
 	}
 }
 
+#ifdef DEBUG
+static int
+gcClearCounts(Hjava_lang_Class *c, void *_)
+{
+	c->live_count = 0;
+	return 0;
+}
+
+static int
+gcDumpCounts(Hjava_lang_Class *c, void *_)
+{
+	if (c->live_count)
+		dprintf("%7d %s\n", c->live_count,	c->name->data);
+	return 0;
+}
+#endif
+
 /*
  * The Garbage Collector sits in a loop starting a collection, waiting
  * until it's finished incrementally, then tidying up before starting
  * another one.
  */
-void
+static void
 gcMan(void* arg)
 {
 	gc_unit* unit;
@@ -463,7 +490,8 @@ DBG(GCSTAT,
 		}
 
 		lockStaticMutex(&gc_lock);
-
+		DBG(GCSTAT, walkClassPool(gcClearCounts, 0));
+		    
 		startGC(gcif);
 
 		for (unit = gclists[grey].cnext; unit != &gclists[grey]; unit = gclists[grey].cnext) {
@@ -492,6 +520,14 @@ DBG(GCSTAT,
 
 		finishGC(gcif);
 
+		DBG(GCSTAT,
+		    dprintf("REACHABLE OBJECT HISTOGRAM\n");
+		    dprintf("%-7s %s\n", "COUNT", "CLASS");
+		    dprintf("%-7s %s\n", "-------",
+			    "-----------------------------------"
+			    "-----------------------------------");
+		    walkClassPool(gcDumpCounts, 0));
+		    
 		unlockStaticMutex(&gc_lock);
 
 		startFinalizer();
@@ -704,7 +740,7 @@ startFinalizer(void)
  * the objects in turn.  An object is only finalised once after which
  * it is deleted.
  */
-void
+static void
 finaliserMan(void* arg)
 {
 	gc_block* info;
@@ -879,7 +915,7 @@ gcMalloc(Collector* gcif, size_t size, int fidx)
 		reserve = gc_primitive_reserve();
 	}
 
-	/* It is not safe to allocate java objects the the first time
+	/* It is not safe to allocate java objects the first time
 	 * gcMalloc is called, but it should be safe after gcEnable
 	 * has been called.
 	 */
@@ -895,13 +931,14 @@ gcMalloc(Collector* gcif, size_t size, int fidx)
 	if (!outOfMem && outOfMem_allocator
 	    && outOfMem_allocator == jthread_current()) { 
 		outOfMem = OOM_ALLOCATING;
-		outOfMem = OutOfMemoryError;
+		outOfMem = OutOfMemoryError; /* implicit allocation */
 		outOfMem_allocator = 0;
 		gc_add_ref(outOfMem);
 	}
 	return (mem);
 }
 
+static
 Hjava_lang_Throwable *
 gcThrowOOM(Collector *gcif)
 {
@@ -926,7 +963,7 @@ gcThrowOOM(Collector *gcif)
 		reserve = 0;
 		if (!ret || ret == OOM_ALLOCATING) {
 			unlockStaticMutex(&gc_lock);
-			ret = OutOfMemoryError;
+			ret = OutOfMemoryError; /* implicit allocation */
 			lockStaticMutex(&gc_lock);
 		}
 	}
@@ -956,7 +993,7 @@ gcRealloc(Collector* gcif, void* mem, size_t size, int fidx)
 	int osize;
 	int iLockRoot;
 
-	assert(fidx == GC_ALLOC_FIXED);
+	assert(gcFunctions[fidx].final == GC_OBJECT_FIXED);
 
 	/* If nothing to realloc from, just allocate */
 	if (mem == NULL) {
