@@ -20,7 +20,7 @@
 #include "support.h"
 #include "stats.h"
 #include "locks.h"
-#include "jthread.h"
+#include "thread.h"
 #include "gc.h"
 #include "gc-mem.h"
 #include "jni.h"
@@ -120,7 +120,7 @@ printslack(void)
 /*
  * check whether the heap is still in a consistent state
  */
-void
+static void
 gc_heap_check(void)
 {
 	int i; 
@@ -491,7 +491,9 @@ DBG(GCFREE,
 		info->size = msz;
 		gc_primitive_free(info);
 	}
-
+DBG(GCDIAG,
+	gc_heap_check();
+    )
 }
 
 /*
