@@ -46,16 +46,22 @@ soft_new(Hjava_lang_Class* c)
 	Hjava_lang_Object* obj;
 	errorInfo info;
 
-	if (processClass(c, CSTATE_COMPLETE, &info) == false) {
-		throwError(&info);
+	if (c->state != CSTATE_COMPLETE && processClass(c, CSTATE_COMPLETE, &info) == false) {
+		goto bad;
+	}
+	obj = newObjectChecked(c, &info);
+	if (obj == 0) {
+		goto bad;
 	}
-	obj = newObject(c);
 
 DBG(NEWINSTR,	
 	dprintf("New object of type %s (%d,%x)\n", 
 		c->name->data, c->bfsize, obj); )
 
 	return (obj);
+bad:
+	throwError(&info);
+	return (0);
 }
 
 /*
@@ -65,12 +71,16 @@ void*
 soft_newarray(jint type, jint size)
 {
 	Hjava_lang_Object* obj;
+	errorInfo info;
 
 	if (size < 0) {
 		throwException(NegativeArraySizeException);
 	}
 
-	obj = newArray(TYPE_CLASS(type), size);
+	obj = newArrayChecked(TYPE_CLASS(type), size, &info);
+	if (obj == 0) {
+		throwError(&info);
+	}
 
 DBG(NEWINSTR,	
 	dprintf("New array of %s [%d] (%x)\n", 
@@ -86,12 +96,16 @@ void*
 soft_anewarray(Hjava_lang_Class* elclass, jint size)
 {
 	Hjava_lang_Object* obj;
+	errorInfo info;
 
 	if (size < 0) {
 		throwException(NegativeArraySizeException);
 	}
 
-	obj = newArray(elclass, size);
+	obj = newArrayChecked(elclass, size, &info);
+	if (obj == 0) {
+		throwError(&info);
+	}
 
 DBG(NEWINSTR,	
 	dprintf("New array object [%d] of %s (%x)\n", size,
@@ -149,9 +163,8 @@ soft_multianewarray(Hjava_lang_Class* class, jint dims, slots* args)
 
 #if defined(TRANSLATOR)
 void*
-soft_multianewarray(Hjava_lang_Class* class, jint dims, ...)
+soft_vmultianewarray(Hjava_lang_Class* class, jint dims, va_list ap)
 {
-	va_list ap;
 	int array[MAXDIMS];
 	int i;
 	jint arg;
@@ -166,11 +179,9 @@ soft_multianewarray(Hjava_lang_Class* class, jint dims, ...)
 	}
 
 	/* Extract the dimensions into an array */
-	va_start(ap, dims);
 	for (i = 0; i < dims; i++) {
 		arg = va_arg(ap, jint);
 		if (arg < 0) {
-			va_end(ap);
 			if (arraydims != array) {
 				KFREE(arraydims);
 			}
@@ -179,7 +190,6 @@ soft_multianewarray(Hjava_lang_Class* class, jint dims, ...)
 		arraydims[i] = arg;
 	}
 	arraydims[i] = -1;
-	va_end(ap);
 
 	/* Mmm, okay now build the array using the wonders of recursion */
         obj = newMultiArray(class, arraydims);
@@ -191,53 +201,69 @@ soft_multianewarray(Hjava_lang_Class* class, jint dims, ...)
 	/* Return the base object */
 	return (obj);
 }
+
+void*
+soft_multianewarray(Hjava_lang_Class* class, jint dims, ...)
+{
+	void* obj;
+	va_list ap;
+	va_start(ap, dims);
+	obj = soft_vmultianewarray(class, dims, ap);
+	va_end(ap);
+	return (obj);
+}
 #endif
 
 /*
- * soft_lookupmethod.
+ * soft_lookupinterfacemethod.
  */
 void*
-soft_lookupmethod(Hjava_lang_Object* obj, Hjava_lang_Class* ifclass, int idx)
+soft_lookupinterfacemethod(Hjava_lang_Object* obj, Hjava_lang_Class* ifclass, int idx)
 {
 	Hjava_lang_Class* cls;
-	Hjava_lang_Class** c;
 	Method* meth;
 	void*	ncode;
-	int i;
-	int dtableidx;
-	errorInfo info;
-
-	if (obj == 0) {
-		throwException(NullPointerException);
-	}
+	register int i;
+	register short* implementors;
 
 	cls = OBJECT_CLASS(obj);
+	implementors = ifclass->implementors;
+	i = cls->impl_index;
+
+#if 1	/* it should never be necessary to initialize the class here
+	 * cause we're invoking a method on an existing, created object
+	 */
+        assert (cls->state >= CSTATE_USABLE);
+#else
 	/* initialize class if necessary */
         if (cls->state < CSTATE_USABLE) {
+		errorInfo info;
 		if (processClass(cls, CSTATE_COMPLETE, &info) == false) {
 			throwError(&info);
 		}
 	}
+#endif
 
-	c = cls->interfaces;
-	i = 0;
-	while (*c != ifclass) {
-		c++;
-		/* do this check inside the loop to avoid loading 
-		 * total_interface_len from memory if we hit the first one. 
-		 */
-		if (++i == cls->total_interface_len) {
-			goto notfound;
-		}
-	}
-
-	dtableidx = cls->itable2dtable[cls->if2itable[i] + idx];
-	if (dtableidx == -1) {
+	/* handle invocations on java.lang.Object methods via INVOKEINTERFACE */
+	if (implementors == 0 || i > implementors[0]) {
 		goto notfound;
 	}
 
-	ncode = *(void**)((char*)cls->dtable 
-			+ DTABLE_METHODOFFSET + dtableidx * DTABLE_METHODSIZE);
+	/* skip word at the beginning of itable2dtable */
+	ncode = cls->itable2dtable[implementors[i] + idx + 1];
+
+	/* This means this class does not implement this interface method 
+	 * at all.  This is something we detect at the time the interface
+	 * dispatch table is built.  To avoid this test, we could instead
+	 * point at a nosuchmethod routine there.  However, we would have
+	 * to find a way to pass along the information which methods it
+	 * is that's missing (or create multiple nosuch_method routines,
+	 * given that they should be rare---minus possible DoS.)
+	 */
+	if (ncode == (void*)0xffffffff) {
+		goto notfound;
+	}
+	assert(ncode);
 	return (ncode);
 
 notfound:
@@ -258,51 +284,19 @@ notfound:
 	 */
 	if (ifclass == ObjectClass) {
 		Method* objm = CLASS_METHODS(ifclass) + idx;
+		errorInfo info;
 		
 		meth = findMethod(cls, objm->name, METHOD_SIG(objm), &info);
 		if (meth == 0) {
 			throwError(&info);
 		}
-		return (METHOD_NATIVECODE(meth));
+		return (METHOD_INDIRECTMETHOD(meth));
 	}
 	meth = CLASS_METHODS(ifclass) + idx;
 	soft_nosuchmethod(cls, meth->name, METHOD_SIG(meth));
 	return (0);
 }
 
-#if 0
-/*
- * soft_lookupinterfacemethod
- * Lookup an interface method, given its name and signature.  Throw exceptions
- * if the method found doesn't fit with wht invokeinterface is expecting.
- */
-void*
-soft_lookupinterfacemethod(Hjava_lang_Class* class, Utf8Const* name, Utf8Const* signature)
-{
-	processClass(class, CSTATE_LINKED);
-	for (; class != 0; class = class->superclass) {
-		Method* mptr = findMethodLocal(class, name, signature);
-		if (mptr != NULL) {
-			if (mptr->accflags & ACC_STATIC) {
-				throwException(IncompatibleClassChangeError);
-			}
-			if (mptr->accflags & ACC_ABSTRACT) {
-				throwException(AbstractMethodError);
-			}
-			if (!(mptr->accflags & ACC_PUBLIC)) {
-				throwException(IllegalAccessError);
-			}
-#if defined(TRANSLATOR)
-			return (METHOD_NATIVECODE(mptr));
-#else
-			return (mptr);
-#endif
-		}
-	}
-	throwException(IncompatibleClassChangeError);
-}
-#endif
-
 inline
 jint
 instanceof_class(Hjava_lang_Class* c, Hjava_lang_Class* oc)
@@ -547,8 +541,13 @@ nativecode*
 soft_fixup_trampoline(FIXUP_TRAMPOLINE_DECL)
 {
 	Method* meth;
+	void **where;
+	void *tramp;
 	errorInfo info;
+
+	/* FIXUP_TRAMPOLINE_INIT sets tramp and where */
 	FIXUP_TRAMPOLINE_INIT;
+	tramp = *where;
 
 	/* If this class needs initializing, do it now.  */
 	if (meth->class->state < CSTATE_USABLE &&
@@ -563,9 +562,17 @@ soft_fixup_trampoline(FIXUP_TRAMPOLINE_DECL)
 		}
 	}
 
-	/* Update dispatch table */
-	if (meth->idx >= 0) {
-		meth->class->dtable->method[meth->idx] = METHOD_NATIVECODE(meth);
+	/* Update the origin of this trampoline --- this could be a 
+	 * dispatch table, interface dispatch table, or a gcj trampoline
+	 *
+	 * This is rather tricky.  We may jump through the same trampoline
+	 * twice.  For instance, class A refers to Double.longBitsToDouble,
+	 * but Double.<clinit> also jumps through the same trampoline
+	 * before it is patched.
+	 */
+	if (*where != *(void**)PMETHOD_NATIVECODE(meth)) {
+		*where = METHOD_NATIVECODE(meth);
+		KFREE(tramp);
 	}
 
 #if 0
