@@ -22,7 +22,7 @@ void*	soft_anewarray(struct Hjava_lang_Class*, jint);
 void	soft_initialise_class(struct Hjava_lang_Class*);
 nativecode* soft_get_method_code (Method*);
 
-void*	soft_lookupmethod(struct Hjava_lang_Object*, struct Hjava_lang_Class*, int);
+void*	soft_lookupinterfacemethod(struct Hjava_lang_Object*, struct Hjava_lang_Class*, int);
 void*	soft_checkcast(struct Hjava_lang_Class*, struct Hjava_lang_Object*);
 jint	soft_instanceof(struct Hjava_lang_Class*, struct Hjava_lang_Object*);
 
