@@ -35,20 +35,21 @@
 #include "config-mem.h"
 #include "config-io.h"
 #include "config-signal.h"
-#include "jtypes.h"
-#include "gtypes.h"
-#include "thread.h"
-#include "support.h"
-#include "md.h"
+#include "gtypes.h"		/* for jlong */
 #include "lerrno.h"
-/* 
- * make sure jsyscall doesn't redefine open, read, etc. 
- * XXX - this is not very clean
- */
-#define NOUNIXPROTOTYPES
-#include "jsyscall.h"
-#define  select(A, B, C, D, E)	\
-	(*Kaffe_SystemCallInterface._select)(A,B,C,D,E)
+#include "support.h"		/* XXX: for currentTime */
+#include "md.h"
+
+#define  NOTIMEOUT                       0
+#if defined (HAVE_SYS_POLL_H) || defined(HAVE_POLL_H)
+#define USE_POLL	1
+#endif
+
+#if defined(__WIN32__)
+#define SIG_T   void(*)()
+#else
+#define SIG_T   void*
+#endif
 
 /*======== 	end of definitions that apply to Kaffe 	     	     ========*/
 
@@ -63,20 +64,7 @@
 
 #endif  /* !KVER */
 
-/*
- * a mutex is represented by a two-word structure
- */
-typedef struct _jmutex
-{
-        struct _jthread *holder;
-        struct _jthread *waiting;
-} jmutex;
-
-/*
- * a condition variable is simply expressed as a list of threads
- * waiting to be notified
- */
-typedef struct _jthread *jcondvar;
+#include "lock-impl.h"
 
 /*
  * This is our internal structure representing the "native" threads.
@@ -96,12 +84,13 @@ typedef struct _jthread {
 	unsigned char			flags;
 	void				(*func)(void *);
 	int				daemon;
+	int				stopCounter;
 
 	/* this one is simply thread specific data, or a cookie -
 	 * used to hold the current Java thread
 	 */
 	void*				jlThread;
-	jmp_buf				env;
+	JTHREAD_JMPBUF			env;
 #if defined(SAVED_FP_SIZE)
 	char				fpstate[SAVED_FP_SIZE];
 #endif
@@ -113,7 +102,7 @@ typedef struct _jthread {
 	double				align[0];
 } jthread, *jthread_t;
 
-#define GET_COOKIE()	(jthread_current()->jlThread)
+extern jthread_t currentJThread;
 
 /****************************************************************************
  *
@@ -124,17 +113,24 @@ typedef struct _jthread {
 /* 
  * initialize the threading system
  */
-jthread_t 
+void 
 jthread_init(
 	int preemptive,			/* preemptive scheduling */
 	int maxpr, 			/* maximum priority */
 	int minpr, 			/* minimum priority */
-	int mainthreadpr,		/* priority of current (main) thread */
-	size_t mainThreadStackSize,	/* assumed main stack size */
         void *(*_allocator)(size_t),	/* memory allocator */
 	void (*_deallocator)(void*),	/* memory deallocator */
 	void (*_destructor1)(void*),	/* called when a thread exits */ 
-	void (*_onstop)());		/* called when a thread is stopped */
+	void (*_onstop)(void), 		/* called when a thread is stopped */
+	void (*_ondeadlock)(void)); 	/* called when we detect deadlock */
+
+/*
+ * Create the first thread - actually bind the first thread to the java
+ * context.
+ */
+jthread_t
+jthread_createfirst(size_t mainThreadStackSize,
+	unsigned char prio, void* jlThread);
 
 /*
  * create a thread with a given priority
@@ -146,11 +142,21 @@ jthread_create(unsigned char pri, 	/* initial priority */
 	void *jlThread, 		/* cookie for this thread */
 	size_t threadStackSize);	/* stack size to be allocated */
 
+struct _exceptionFrame;
+typedef void (*exchandler_t)(struct _exceptionFrame*);
+
+/*
+ * Register handlers for null pointer accesses and floating point exceptions
+ */
+void
+jthread_initexceptions(exchandler_t _nullHandler,
+                       exchandler_t _floatingHandler);
+
 /*
  * set a function to be run when last non-daemon dies 
  * this is used to run the finalizer on exit.
  */
-void 	jthread_atexit(void (*f)());
+void 	jthread_atexit(void (*f)(void));
 
 /*
  * walk all live threads, and invoke `func', passing in their cookie
@@ -171,7 +177,7 @@ void	jthread_setpriority(jthread_t jtid, int prio);
 /*
  * yield to another thread
  */
-void 	jthread_yield();
+void 	jthread_yield(void);
 
 /*
  * sleep for time milliseconds
@@ -182,9 +188,8 @@ void 	jthread_sleep(jlong time);
  * return the current thread 
  */
 static inline jthread_t
-jthread_current() 
+jthread_current(void) 
 { 
-	extern jthread_t currentJThread;
 	return currentJThread; 
 }
 
@@ -211,62 +216,144 @@ void 	jthread_interrupt(jthread_t jtid);
 /*
  * have the current thread exit
  */
-void 	jthread_exit();
+void 	jthread_exit(void) __NORETURN__;
 
 /*
  * determine whether a location is on the stack of the current thread
- * (was FRAMEOKAY)
  */
-int 	jthread_on_current_stack(void *bp);
+static inline int
+jthread_on_current_stack(void *bp)      
+{
+        int rc = bp >= currentJThread->stackBase && bp < currentJThread->stackEnd;
+
+DBG(JTHREADDETAIL,
+	dprintf("on current stack: base=%p size=%d bp=%p %s\n",
+		currentJThread->stackBase,
+		currentJThread->stackEnd-currentJThread->stackBase,
+		bp,
+		(rc ? "yes" : "no"));
+    )
+
+        return rc;
+}
+
+/* 
+ * Check for room on stack.
+ */
+static inline int     
+jthread_stackcheck(int left)
+{
+	int rc;
+#if defined(STACK_GROWS_UP)
+        rc = jthread_on_current_stack((char*)&rc + left);
+#else
+        rc = jthread_on_current_stack((char*)&rc - left);
+#endif
+	return (rc);
+}
+
+/*
+ * Get the current stack limit.
+ */
+
+#define	REDZONE	1024
+
+static
+inline
+void*
+jthread_stacklimit(void)
+{
+#if defined(STACK_GROWS_UP)
+        return (void*)((uintp)currentJThread->stackEnd - REDZONE);
+#else
+        return (void*)((uintp)currentJThread->stackBase + REDZONE);
+#endif
+}
 
 /*
  * determine the "interesting" stack range a conservative gc must walk
  */
-void jthread_extract_stack(jthread_t jtid, void **from, unsigned *len);
+int jthread_extract_stack(jthread_t jtid, void **from, unsigned *len);
 
 /*
  * Disallow cancellation for current thread
  */
-void jthread_disable_stop();
+void jthread_disable_stop(void);
 
 /*
  * Reallow cancellation for current thread
  * If a cancellation is pending, the stop method will be called
  */
-void jthread_enable_stop();
+void jthread_enable_stop(void);
 
 /*
  * functions to disable and restore interrupts
+ * These are *not* part of the public interface.
  */
-void 	intsDisable();
-void 	intsRestore();
-void	intsRestoreAll();
-int 	intsDisabled();
+#define	INTS_DISABLED()		intsDisabled()
+int 	intsDisabled(void);
+
+/*
+ * Prevent all other threads from running.
+ * In this uniprocessor implementation, this is simple.
+ */
+void jthread_suspendall(void);
+
+/*
+ * Reallow other threads.
+ * In this uniprocessor implementation, this is simple.
+ */
+void jthread_unsuspendall(void);
+
+/*
+ * Print info about a given jthread to stderr
+ */
+void jthread_dumpthreadinfo(jthread_t tid);
+
+/*
+ * return thread-specific data for a given jthread
+ */
+static inline void*
+jthread_getcookie(jthread_t tid) 
+{
+	return (tid->jlThread);
+}
 
 /*
  * API related to I/O
  */
-int jthreadedFileDescriptor(int fd);
-int jthreadedOpen(const char* path, int flags, int mode);
-int jthreadedSocket(int af, int type, int proto);
-int jthreadedConnect(int fd, struct sockaddr* addr, size_t len);
-int jthreadedAccept(int fd, struct sockaddr* addr, size_t* len);
-ssize_t jthreadedRead(int fd, void* buf, size_t len);
-ssize_t jthreadedWrite(int fd, const void* buf, size_t len);
-ssize_t jthreadedRecvfrom(int fd, void* buf, size_t len, int flags,
-        struct sockaddr* from, int* fromlen);
-int jthreadedWaitpid(int wpid, int* status, int options);
+int jthreadedOpen(const char* path, int flags, int mode, int*);
+int jthreadedSocket(int af, int type, int proto, int*);
+int jthreadedConnect(int fd, struct sockaddr* addr, int len, int timeout);
+int jthreadedAccept(int fd, struct sockaddr* addr, int* len, 
+	int timeout, int *);
+int jthreadedRead(int fd, void* buf, size_t len, ssize_t *);
+int jthreadedTimedRead(int fd, void* buf, size_t len, int timeout, ssize_t *);
+int jthreadedWrite(int fd, const void* buf, size_t len, ssize_t *);
+int jthreadedRecvfrom(int fd, void* buf, size_t len, int flags,
+        struct sockaddr* from, int* fromlen, int timeout, ssize_t *);
+int jthreadedWaitpid(int wpid, int* status, int options, int *);
+int jthreadedForkExec(char **argv, char **arge,
+	int ioes[4], int *, const char *);
 
-/* 
- * Locking API
- */
-void jmutex_initialise(jmutex *lock);
-void jmutex_lock(jmutex *lock);
-void jmutex_unlock(jmutex *lock);
+/* restore an fd, i.e., put it in blocking state without async I/O */
+#define JTHREAD_RESTORE_FD
+void jthreadRestoreFD(int fd);
 
-void jcondvar_initialise(jcondvar *cv);
-void jcondvar_wait(jcondvar *cv, jmutex *lock, jlong timeout);
-void jcondvar_signal(jcondvar *cv, jmutex *lock);
-void jcondvar_broadcast(jcondvar *cv, jmutex *lock);
+
+/* Spinlocks: simple since we're uniprocessor */
+/* ARGSUSED */
+static inline
+void jthread_spinon(void *arg)
+{
+	jthread_suspendall();
+}
+
+/* ARGSUSED */
+static inline
+void jthread_spinoff(void *arg)
+{
+	jthread_unsuspendall();
+}
 
 #endif
