@@ -4,7 +4,7 @@
  * Copyright (c) 1996, 1997
  *	Transvirtual Technologies, Inc.  All rights reserved.
  *
- * See the file "lib-license.terms" for information on usage and redistribution 
+ * See the file "license.terms" for information on usage and redistribution 
  * of this file. 
  */
 
@@ -18,40 +18,53 @@
 #include "../../../kaffe/kaffevm/classMethod.h"
 #include "../../../kaffe/kaffevm/itypes.h"
 #include "../../../kaffe/kaffevm/support.h"
+#include "../../../kaffe/kaffevm/stringSupport.h"
 #include "../../../kaffe/kaffevm/soft.h"
 #include "../../../kaffe/kaffevm/stackTrace.h"
-#include "InputStream.h"
-#include "PrintStream.h"
-#include "System.h"
-#include "Constructor.h"
-#include "Method.h"
-#include "Field.h"
-#include <native.h>
+#include "../../../kaffe/kaffevm/baseClasses.h"
+#include "../../../kaffe/kaffevm/lookup.h"
+#include "java_io_InputStream.h"
+#include "java_io_PrintStream.h"
+#include "java_lang_System.h"
+#include "java_lang_reflect_Constructor.h"
+#include "java_lang_reflect_Method.h"
+#include "java_lang_reflect_Field.h"
+#include "java_lang_Class.h"
 #include "defs.h"
 
+extern Hjava_lang_Object* buildStackTrace(struct _exceptionFrame*);
+
 /*
  * Convert string name to class object.
  */
 struct Hjava_lang_Class*
-java_lang_Class_forName(struct Hjava_lang_String* str)
+java_lang_Class_forName(struct Hjava_lang_String* str, jbool doinit,
+	Hjava_lang_ClassLoader* loader)
 {
+	errorInfo einfo;
 	Hjava_lang_Class* clazz;
-	Hjava_lang_Class* loadedclass;
-	Hjava_lang_ClassLoader* loader;
-	char buf[MAXNAMELEN];
-	int depth;
-
-	/* Get string and convert '.' to '/' */
-	javaString2CString(str, buf, sizeof(buf));
-	classname2pathname(buf, buf);
+	Utf8Const *utf8buf;
+	const char *buf;
+	int jlen;
+	jchar *js;
 
 	/*
-	 * If the calling method is in a class that was loaded by a class
-	 * loader, use that class loader to find the class corresponding to
-	 * the name.  Otherwise, use the system class loader.
+	 * NB: internally, we store class names as path names (with slashes
+	 *     instead of dots.  However, we must also prevent calls to
+	 *     "java/lang/Object" or "[[Ljava/lang/Object;" from succeeding.
+	 *	Since class names cannot have slashes, we reject all attempts
+	 *	to look up names that do.  Awkward.  Inefficient.
 	 */
-	loadedclass = getClassWithLoader(&depth);
-	loader = (depth == 0) ? loadedclass->loader : NULL;
+	js = STRING_DATA(str);
+	jlen = STRING_SIZE(str);
+	while (--jlen > 0) {
+		if (*js++ == '/') {
+			postExceptionMessage(&einfo, 
+				JAVA_LANG(ClassNotFoundException), 
+				"Cannot have slashes - use dots instead.");
+			throwError(&einfo);
+		}
+	}
 
 	/*
 	 * Note the following oddity: 
@@ -66,63 +79,91 @@ java_lang_Class_forName(struct Hjava_lang_String* str)
 	 *
 	 * This is somewhat described in Section 5.1.3 of the VM 
 	 * Specification, titled "Array Classes".  This section seems to 
-	 * imply that we must avoid to ask a class loader to resolve such
+	 * imply that we must avoid asking a class loader to resolve such
 	 * array names (those starting with an [), and this is what calling
 	 * loadArray does.
 	 */
-	clazz = ((buf[0] == '[') ? loadArray : loadClass)
-			(makeUtf8Const (buf, strlen(buf)), loader);
 
-	/* if we got here, either clazz must be valid or a 
-	 * ClassNotFoundException was thrown
-	 */
-	assert(clazz != 0);
+	/* Convert string to utf8, converting '.' to '/' */
+	utf8buf = checkPtr(stringJava2Utf8ConstReplace(str, '.', '/'));
+	buf = utf8buf->data;
+
+	if (buf[0] == '[') {
+		clazz = loadArray(utf8buf, loader, &einfo);
+	}
+	else {
+		clazz = loadClass(utf8buf, loader, &einfo);
+	}
+
+	/* if an error occurred, throw an exception */
+	if (clazz == 0) {
+		/* The only checked exception that Class.forName() throws
+		 * is ClassNotFoundException.  This is an exception, not an
+		 * Error, which users often catch.
+		 *
+		 * However, Class.forName() can also throw errors, such as
+		 * NoClassDefFoundError, if for instance a superclass for
+		 * a class couldn't be found.
+		 *
+		 * When it throws which, we don't really know.  We try to be 
+		 * compatible, so we upgrade the error to an exception if it's 
+		 * (NoClassDefFoundError, this_class_name), or if it's a
+		 * VerifyError.
+		 * NB: 1.2 seems to be more consistent and throws 
+		 * ClassNotFoundException in most cases.
+		 */
+		if ((einfo.type & KERR_EXCEPTION) && !strcmp(einfo.classname, "java.lang.VerifyError")) 
+		{
+			errorInfo einfo_copy = einfo;
+			postExceptionMessage(&einfo, 
+				JAVA_LANG(ClassNotFoundException), 
+				einfo.mess);
+			discardErrorInfo(&einfo_copy);
+		} else
+		if ((einfo.type & KERR_EXCEPTION) && !strcmp(einfo.classname, "java.lang.NoClassDefFoundError"))
+		{
+			/*
+			 * However, we don't upgrade if it is a second attempt
+			 * to load a class whose loading has already failed.
+			 */
+			classEntry* centry;
+			centry = lookupClassEntry(utf8buf, loader, &einfo);
+			if (centry == 0 || (centry->class &&
+			    centry->class->state == CSTATE_FAILED)) {
+				utf8ConstRelease(utf8buf);
+				throwError(&einfo);
+			}
+
+			/* This is not quite what Sun does: they use the
+			 * classname, we use the pathname as the message
+			 * of the exception  (FIXME?)
+			 */
+			if (buf[0] == '[' || !strcmp(einfo.mess, buf)) {
+				errorInfo einfo_copy = einfo;
+				postExceptionMessage(&einfo,
+					JAVA_LANG(ClassNotFoundException), 
+					einfo.mess);
+				discardErrorInfo(&einfo_copy);
+			}
+		}
+		utf8ConstRelease(utf8buf);
+		throwError(&einfo);
+	}
+	utf8ConstRelease(utf8buf);
 
 	/*
-	 * Note:
-	 * The JDK documentation says: "...this method attempts to locate, 
-	 * load and link the class."
-	 *
 	 * loadClass returns the class in state CSTATE_LINKED.
-	 * Processing to CSTATE_OK will initialize the class, resolve
+	 *
+	 * Processing to CSTATE_COMPLETE will initialize the class, resolve
 	 * its constants and run its static initializers.
 	 *
-	 * According to 2.16.4 of the VM Specification, a class should be
-	 * initialized at its first active use.   The VM Specification
-	 * does not mention whether Class.forName is an active use, and
-	 * although 2.16.4 seems to imply that it is not, Sun's implementation 
-	 * seems to run the static initializerss of a class when 
-	 * Class.forName is called (see forNameTest.java)
-	 *
-	 * On the other hand, processing the class to CSTATE_OK is bad
-	 * because our constant resolution will require that all classes
-	 * to which the class refers are resolvable.  In practice, this
-	 * causes ClassNotFoundExceptions to be thrown in situations where
-	 * Sun would not throw such an exception, for instance, if serialver
-	 * finds that .class files are missing.  This should be tolerable.
-	 *
-	 * Also, 2.16.4 of the VM spec says:
-	 *
-	 *  "The intent here is that a type has a set of initializers that 
-	 *   put it in a consistent state, and that this state is the first 
-	 *   state that is observed by other classes".
-	 * 
-	 * We are somewhat inconsistent here.  The following list lists
-	 * the functions that return or process Class objects to or for
-	 * the user, and the state in which the class is afterwards:
-	 *
-	 *	ClassLoader_defineClass0: 	returns CSTATE_PREPARED
-	 * 	ClassLoader_resolveClass0:	returns CSTATE_LINKED
-	 * 	ClassLoader_findSystemClass0:	returns CSTATE_LINKED
-	 * 	Class_forName:			returns CSTATE_OK
-	 *
-	 * We have also observed that processing a class to CSTATE_OK in
-	 * defineClass0 *does not* work; it causes repeated invocations of
-	 * static initializers, which crashes in classMethod.c where
-	 * ncode_start and ncode_end are set to zero after a initializer
-	 * was invoked.
+	 * The option to load a class via forName without initializing it
+	 * was introduced in 1.2, presumably for the convenience of
+	 * programs such as stub compilers.
 	 */
-	processClass(clazz, CSTATE_OK);
+	if (doinit && processClass(clazz, CSTATE_COMPLETE, &einfo) == false) {
+		throwError(&einfo);
+	}
 	return (clazz);
 }
 
@@ -132,29 +173,7 @@ java_lang_Class_forName(struct Hjava_lang_String* str)
 struct Hjava_lang_String*
 java_lang_Class_getName(struct Hjava_lang_Class* c)
 {
-	char buffer[100];
-	struct Hjava_lang_String* str;
-	int len;
-	char* name;
-	int i;
-	char* ptr;
-	char ch;
-
-	len = c->name->length;
-	name = len > 100 ? (char*)malloc(len) : buffer;
-	ptr = c->name->data;
-	for (i = 0; i < len; i++) {
-		ch = *ptr++;
-		if (ch == '/') {
-			ch = '.';
-		}
-		name[i] = ch;
-	}
-	str = makeJavaString(name, len);
-	if (name != buffer) {
-		free (name);
-	}
-	return (str);
+	return(checkPtr(utf8Const2JavaReplace(c->name, '/', '.')));
 }
 
 /*
@@ -163,7 +182,11 @@ java_lang_Class_getName(struct Hjava_lang_Class* c)
 struct Hjava_lang_Object*
 java_lang_Class_newInstance(struct Hjava_lang_Class* this)
 {
-	return (execute_java_constructor(0, this, "()V"));
+	if (CLASS_IS_PRIMITIVE(this)) {
+		SignalError("java.lang.InstantiationException", 
+			    CLASS_CNAME(this));
+	}
+	return (execute_java_constructor(0, 0, this, "()V"));
 }
 
 /*
@@ -196,14 +219,18 @@ java_lang_Class_getInterfaces(struct Hjava_lang_Class* this)
 	int nr;
 
 	nr = this->interface_len;
-	/*
-	 * Do not report java.io.Serializable for array classes
-	 */
-	if (CLASS_IS_ARRAY(this))
-		nr = 0;
 
-	obj = (HArrayOfObject*)AllocObjectArray(nr, "Ljava/lang/Class");
-	ifaces = (struct Hjava_lang_Class**)unhand(obj)->body;
+#if defined(JDK_1_1_COMPAT)
+	/*
+	 * Do not report java.io.Serializable for array classes in JDK 1.1
+	 */
+	if (CLASS_IS_ARRAY(this)) {
+		nr = 0;
+	}
+#endif
+
+	obj = (HArrayOfObject*)AllocObjectArray(nr, "Ljava/lang/Class;", 0);
+	ifaces = (struct Hjava_lang_Class**)unhand_array(obj)->body;
 	for (i = 0; i < nr; i++) {
 		ifaces[i] = this->interfaces[i];
 	}
@@ -215,7 +242,7 @@ java_lang_Class_getInterfaces(struct Hjava_lang_Class* this)
  * Return the class loader which loaded me.
  */
 struct Hjava_lang_ClassLoader*
-java_lang_Class_getClassLoader(struct Hjava_lang_Class* this)
+java_lang_Class_getClassLoader0(struct Hjava_lang_Class* this)
 {
 	return (this->loader);
 }
@@ -244,8 +271,6 @@ java_lang_Class_isArray(struct Hjava_lang_Class* this)
 Hjava_lang_Class*
 java_lang_Class_getComponentType(struct Hjava_lang_Class* this)
 {
-	Hjava_lang_Class* type;
-
 	if (CLASS_IS_ARRAY(this)) {
 		return (CLASS_ELEMENT_TYPE(this));
 	}
@@ -268,7 +293,7 @@ java_lang_Class_getPrimitiveClass(struct Hjava_lang_String* name)
 {
 	jchar* chrs;
 
-	chrs = &unhand(unhand(name)->value)->body[unhand(name)->offset];
+	chrs = STRING_DATA(name);
 	switch (chrs[0]) {
 	case 'b':
 		if (chrs[1] == 'y') {
@@ -323,49 +348,39 @@ java_lang_Class_setSigners(struct Hjava_lang_Class* this, HArrayOfObject* sigs)
 	unimp("java.lang.Class:setSigners unimplemented");
 }
 
-static
-HArrayOfObject*
+static HArrayOfObject*
 makeParameters(Method* meth)
 {
 	int i;
-	int len;
-	char* sig;
 	HArrayOfObject* array;
+	errorInfo info;
 	Hjava_lang_Class* clazz;
 
-	sig = meth->signature->data;
-	len = 0;
-	for (sig++; *sig != ')'; sig++, len++) {
-		while (*sig == '[') {
-			sig++;
+	array = (HArrayOfObject*)AllocObjectArray(METHOD_NARGS(meth),
+	    "Ljava/lang/Class;", 0);
+	for (i = 0; i < METHOD_NARGS(meth); ++i) {
+		clazz = getClassFromSignaturePart(METHOD_ARG_TYPE(meth, i),
+					      meth->class->loader, &info);
+		if (clazz == 0) {
+			throwError(&info);
 		}
-		if (*sig == 'L') {
-			while (*sig != ';') {
-				sig++;
-			}
-		}
-	}
-
-	array = (HArrayOfObject*)AllocObjectArray(len, "Ljava/lang/Class;");
-	sig = meth->signature->data + 1;	/* Skip leading '(' */
-	for (i = 0; i < len; i++) {
-		clazz = classFromSig(&sig, meth->class->loader);
-		unhand(array)->body[i] = &clazz->head;
+		unhand_array(array)->body[i] = &clazz->head;
 	}
 
         return (array);
 }
 
-static
-Hjava_lang_Class*
+static Hjava_lang_Class*
 makeReturn(Method* meth)
 {
-	char* sig;
+	errorInfo info;
+	Hjava_lang_Class* clazz;
 
-	/* Skip to end of signature to find return type */
-	sig = strchr(meth->signature->data, ')') + 1;
-
-	return (classFromSig(&sig, meth->class->loader));
+	clazz = getClassFromSignaturePart(METHOD_RET_TYPE(meth), meth->class->loader, &info);
+	if (clazz == 0) {
+		throwError(&info);
+	}
+	return (clazz);
 }
 
 /*
@@ -375,8 +390,7 @@ makeReturn(Method* meth)
  *
  * We do not bother to cache the resolved types here.
  */
-static
-HArrayOfObject*
+static HArrayOfObject*
 makeExceptions(Method* meth)
 {
 	int nr;
@@ -385,10 +399,17 @@ makeExceptions(Method* meth)
 	Hjava_lang_Class** ptr;
 
 	nr = meth->ndeclared_exceptions;
-	array = (HArrayOfObject*)AllocObjectArray(nr, "Ljava/lang/Class;");
-	ptr = (Hjava_lang_Class**)&unhand(array)->body[0];
+	array = (HArrayOfObject*)AllocObjectArray(nr, "Ljava/lang/Class;", 0);
+	ptr = (Hjava_lang_Class**)&unhand_array(array)->body[0];
 	for (i = 0; i < nr; i++) {
-		*ptr++ = getClass(meth->declared_exceptions[i], meth->class);
+		errorInfo info;
+		Hjava_lang_Class* clazz;
+		clazz = getClass(meth->declared_exceptions[i], meth->class, 
+				&info);
+		if (clazz == 0) {
+			throwError(&info);
+		}
+		*ptr++ = clazz;
 	}
 	return (array);
 }
@@ -401,7 +422,8 @@ makeConstructor(struct Hjava_lang_Class* clazz, int slot)
 	Method* mth;
 
 	mth = CLASS_METHODS(clazz) + slot;
-	meth = (Hjava_lang_reflect_Constructor*)AllocObject("java/lang/reflect/Constructor");      
+	meth = (Hjava_lang_reflect_Constructor*)
+	    AllocObject("java/lang/reflect/Constructor", 0);      
 
 	unhand(meth)->clazz = clazz;
 	unhand(meth)->slot = slot;
@@ -419,12 +441,12 @@ makeMethod(struct Hjava_lang_Class* clazz, int slot)
 	Method* mth;
 
 	mth = CLASS_METHODS(clazz) + slot;
-	meth = (Hjava_lang_reflect_Method*)AllocObject("java/lang/reflect/Method");      
+	meth = (Hjava_lang_reflect_Method*)
+	    AllocObject("java/lang/reflect/Method", 0);      
 
 	unhand(meth)->clazz = clazz;
 	unhand(meth)->slot = slot;
-	unhand(meth)->name = makeReplaceJavaStringFromUtf8(
-		mth->name->data, mth->name->length, 0, 0);
+	unhand(meth)->name = checkPtr(utf8Const2Java(mth->name));
 	unhand(meth)->parameterTypes = makeParameters(mth);
 	unhand(meth)->exceptionTypes = makeExceptions(mth);
 	unhand(meth)->returnType = makeReturn(mth);
@@ -438,30 +460,64 @@ makeField(struct Hjava_lang_Class* clazz, int slot)
 {
 	Hjava_lang_reflect_Field* field;
 	Field* fld;
+	errorInfo info;
 
 	fld = CLASS_FIELDS(clazz) + slot;
-	field = (Hjava_lang_reflect_Field*)AllocObject("java/lang/reflect/Field");
+	field = (Hjava_lang_reflect_Field*)
+	    AllocObject("java/lang/reflect/Field", 0);
 	unhand(field)->clazz = clazz;
 	unhand(field)->slot = slot;
-	unhand(field)->type = (struct Hjava_lang_Class*)fld->type;
-	unhand(field)->name = makeReplaceJavaStringFromUtf8(
-		fld->name->data, fld->name->length, 0, 0);
+	unhand(field)->type = resolveFieldType(fld, clazz, &info);
+	if (unhand(field)->type == 0) {
+		throwError(&info);
+	}
+	unhand(field)->name = checkPtr(utf8Const2Java(fld->name));
 	return (field);
 }
  
+/* 
+ * Return true if there is method defined in any subclass of 'cls'
+ * that overrides 'meth'.
+ *
+ * Assumes that base is a subclass of cls
+ */
+static int
+isOverridden(Hjava_lang_Class *base, Hjava_lang_Class *cls, Method *meth)
+{
+	/* XXX for interfaces for now */
+	if (base == 0) 
+		return (false);
+
+	/* Search superclasses for equivalent method name.
+	 * If found extract its index nr.
+	 */
+	for (; base != cls;  base = base->superclass) {
+		int j = CLASS_NMETHODS(base);
+		Method* mt = CLASS_METHODS(base);
+		for (; --j >= 0;  ++mt) {
+			if (utf8ConstEqual (mt->name, meth->name) &&
+			    utf8ConstEqual (METHOD_SIG(mt), METHOD_SIG(meth)))
+			{
+				return (true);
+			}
+		}
+	}
+	return (false);
+}
+
 /*
  * count the number of methods in a class that are not constructors.
  * If declared is not set, count only public methods.
  */
 static int
-countMethods(struct Hjava_lang_Class* clas, jint declared)
+countMethods(Hjava_lang_Class* base, Hjava_lang_Class* clas, jint declared)
 {
 	Method* mth = CLASS_METHODS(clas);
 	int i;
 	int count = 0;
 
 	for (i = CLASS_NMETHODS(clas)-1 ; i >= 0; i--) {
-		if (((mth[i].accflags & ACC_PUBLIC) || declared) && !(mth[i].accflags & ACC_CONSTRUCTOR)) {
+		if (((mth[i].accflags & ACC_PUBLIC) || declared) && !(mth[i].accflags & ACC_CONSTRUCTOR) && !isOverridden(base, clas, mth + i)) {
 			count++;
 		}
 	}
@@ -473,14 +529,14 @@ countMethods(struct Hjava_lang_Class* clas, jint declared)
  * not constructors.  If declared is not set, include only public methods.
  */
 static void
-addMethods(struct Hjava_lang_Class* clas, jint declared, 
+addMethods(Hjava_lang_Class* base, Hjava_lang_Class* clas, jint declared, 
 	Hjava_lang_reflect_Method*** ptr)
 {
 	Method* mth = CLASS_METHODS(clas);
 	int i;
 
 	for (i = CLASS_NMETHODS(clas)-1; i >= 0; i--) {
-		if (((mth[i].accflags & ACC_PUBLIC) || declared) && !(mth[i].accflags & ACC_CONSTRUCTOR)) {
+		if (((mth[i].accflags & ACC_PUBLIC) || declared) && !(mth[i].accflags & ACC_CONSTRUCTOR) && !isOverridden(base, clas, mth + i)) {
 			**ptr = makeMethod(clas, i);
 			(*ptr)++;
 		}
@@ -510,20 +566,21 @@ getInterfaceMethods0(struct Hjava_lang_Class* this, jint declared)
 	int i;
 
 	count = 0;
-	count += countMethods(this, declared);
+	count += countMethods(0, this, declared);
 	if (!declared) {
 		for (i = 0; i < this->total_interface_len; i++) {
-			count += countMethods(this->interfaces[i], declared);
+			count += countMethods(0, this->interfaces[i], declared);
 		}
 	}
 
-	array = (HArrayOfObject*)AllocObjectArray(count, "Ljava/lang/reflect/Method;");
-	ptr = (Hjava_lang_reflect_Method**)&unhand(array)->body[0];
+	array = (HArrayOfObject*)
+	    AllocObjectArray(count, "Ljava/lang/reflect/Method;", 0);
+	ptr = (Hjava_lang_reflect_Method**)&unhand_array(array)->body[0];
 
-	addMethods(this, declared, &ptr);
+	addMethods(0, this, declared, &ptr);
 	if (!declared) {
 		for (i = 0; i < this->total_interface_len; i++) {
-			addMethods(this->interfaces[i], declared, &ptr);
+			addMethods(0, this->interfaces[i], declared, &ptr);
 		}
 	}
 
@@ -531,14 +588,12 @@ getInterfaceMethods0(struct Hjava_lang_Class* this, jint declared)
 }
 
 HArrayOfObject*
-java_lang_Class_getMethods0(struct Hjava_lang_Class* this, jint declared)
+java_lang_Class_getMethods0(struct Hjava_lang_Class* this, jboolean declared)
 {
 	int count;
 	Hjava_lang_Class* clas;
-	Method* mth;
 	Hjava_lang_reflect_Method** ptr;
 	HArrayOfObject* array;
-	int i;
 
 	/*
 	 * Note: the spec wants us to include the methods of all superclasses
@@ -553,17 +608,18 @@ java_lang_Class_getMethods0(struct Hjava_lang_Class* this, jint declared)
 	count = 0;
 	for (clas = this; clas != NULL; clas = clas->superclass) {
 
-		count += countMethods(clas, declared);
+		count += countMethods(this, clas, declared);
 
 		if (declared) {
 			break;
 		}
 	}
-	array = (HArrayOfObject*)AllocObjectArray(count, "Ljava/lang/reflect/Method;");
-	ptr = (Hjava_lang_reflect_Method**)&unhand(array)->body[0];
+	array = (HArrayOfObject*)
+	    AllocObjectArray(count, "Ljava/lang/reflect/Method;", 0);
+	ptr = (Hjava_lang_reflect_Method**)&unhand_array(array)->body[0];
 	for (clas = this; clas != NULL; clas = clas->superclass) {
 
-		addMethods(clas, declared, &ptr);
+		addMethods(this, clas, declared, &ptr);
 
 		if (declared) {
 			break;
@@ -574,7 +630,7 @@ java_lang_Class_getMethods0(struct Hjava_lang_Class* this, jint declared)
 }
 
 HArrayOfObject*
-java_lang_Class_getConstructors0(struct Hjava_lang_Class* this, jint declared)
+java_lang_Class_getConstructors0(struct Hjava_lang_Class* this, jboolean declared)
 {
 	int count;
 	Hjava_lang_Class* clas;
@@ -591,8 +647,9 @@ java_lang_Class_getConstructors0(struct Hjava_lang_Class* this, jint declared)
 			count++;
 		}
 	}
-	array = (HArrayOfObject*)AllocObjectArray(count, "Ljava/lang/reflect/Constructor;");
-	ptr = (Hjava_lang_reflect_Constructor**)&unhand(array)->body[0];
+	array = (HArrayOfObject*)
+	   AllocObjectArray(count, "Ljava/lang/reflect/Constructor;", 0);
+	ptr = (Hjava_lang_reflect_Constructor**)&unhand_array(array)->body[0];
 	clas = this;
 	mth = CLASS_METHODS(clas);
 	for (i = CLASS_NMETHODS(clas)-1; i >= 0;  i--) {
@@ -604,34 +661,41 @@ java_lang_Class_getConstructors0(struct Hjava_lang_Class* this, jint declared)
 	return (array);
 }
 
-HArrayOfObject*
-java_lang_Class_getFields0(struct Hjava_lang_Class* clazz, int declared)
+/*
+ * Count all public fields of a class, including inherited fields and
+ * fields from implemented interfaces
+ */
+static int
+countPublicFields(Hjava_lang_Class* clazz) 
 {
-	int count;
 	Hjava_lang_Class* clas;
-	Field* fld;
-	Hjava_lang_reflect_Field** ptr;
-	HArrayOfObject* array;
-	int i;
+	int i, count;
 
-	if (declared) {
-		count = CLASS_NFIELDS((Hjava_lang_Class*)clazz);
-	}
-	else {
-		count = 0;
-		for (clas = clazz; clas != NULL; clas = clas->superclass) {
-			fld = CLASS_FIELDS(clas);
-			for (i = CLASS_NFIELDS(clas)-1; i >= 0; i--) {
-				if (fld[i].accflags & ACC_PUBLIC) {
-					count++;
-				}
+	count = 0;
+	for (clas = clazz; clas != NULL; clas = clas->superclass) {
+		Field *fld = CLASS_FIELDS(clas);
+		for (i = CLASS_NFIELDS(clas)-1; i >= 0; i--) {
+			if (fld[i].accflags & ACC_PUBLIC) {
+				count++;
 			}
 		}
 	}
-	array = (HArrayOfObject*)AllocObjectArray(count, "Ljava/lang/reflect/Field;");
-	ptr = (Hjava_lang_reflect_Field**)&unhand(array)->body[0];
+
+	for (i = 0; i < clazz->total_interface_len; i++) {
+		count += countPublicFields(clazz->interfaces[i]);
+	}
+	return (count);
+}
+
+static void
+makePublicFields(Hjava_lang_Class* clazz, jboolean declared, Hjava_lang_reflect_Field*** pptr)
+{
+	Hjava_lang_Class* clas;
+	Hjava_lang_reflect_Field** ptr = *pptr;
+	int i;
+
 	for (clas = clazz; clas != NULL; clas = clas->superclass) {
-		fld = CLASS_FIELDS(clas);
+		Field *fld = CLASS_FIELDS(clas);
 		for (i = CLASS_NFIELDS(clas)-1; i >= 0;  i--) {
 			if ((fld[i].accflags & ACC_PUBLIC) || declared) {
 				*ptr = makeField(clas, i);
@@ -642,6 +706,40 @@ java_lang_Class_getFields0(struct Hjava_lang_Class* clazz, int declared)
 			break;
 		}
 	}
+	if (!declared) {
+		for (i = 0; i < clazz->total_interface_len; i++) {
+			makePublicFields(clazz->interfaces[i], declared, &ptr);
+		}
+	}
+	*pptr = ptr;
+}
+
+/* 
+ * Below, "declared" means to include a field only if it is directly
+ * declared by that class (and not inherited from a superclass or defined
+ * by an interface the class implements.)  This applies to both private,
+ * protected, and public fields.
+ *
+ * On the other hand, if "declared" is false, we only include public 
+ * fields.  Weird semantics.
+ */
+HArrayOfObject*
+java_lang_Class_getFields0(struct Hjava_lang_Class* clazz, jboolean declared)
+{
+	int count;
+	Hjava_lang_reflect_Field** ptr;
+	HArrayOfObject* array;
+
+	if (declared) {
+		count = CLASS_NFIELDS((Hjava_lang_Class*)clazz);
+	}
+	else {
+		count = countPublicFields(clazz);
+	}
+	array = (HArrayOfObject*)
+	    AllocObjectArray(count, "Ljava/lang/reflect/Field;", 0);
+	ptr = (Hjava_lang_reflect_Field**)&unhand_array(array)->body[0];
+	makePublicFields(clazz, declared, &ptr);
 
 	return (array);
 }
@@ -662,52 +760,96 @@ static
 int
 checkParameters(Method* mth, HArrayOfObject* argtypes)
 {
-	char *sig = mth->signature->data;
-	int   i;
+	int i;
+	errorInfo info;
 
-	sig++;	/* skip leading '(' */
-	for (i = 0; i < ARRAY_SIZE(argtypes); i++) {
-
-		/* signature was too short or type doesn't match */
-		if (!*sig || (struct Hjava_lang_Class *)
-				OBJARRAY_DATA(argtypes)[i] != 
-				classFromSig(&sig, mth->class->loader))
-			return (0);
+	/* The JDK doc says and experimentation shows that a null second 
+	 * parameter to all get(Declared){Method|Constructor} functions
+	 * is treated like passing an empty array "new Class [] {}"
+	 */
+	if ((argtypes == NULL) && METHOD_NARGS(mth) == 0) {
+		return (1);
 	}
-	/* return false if signature was too long */
-	return (*sig == ')' ? 1 : 0);
+
+	if (ARRAY_SIZE(argtypes) != METHOD_NARGS(mth)) {
+		return (0);
+	}
+
+	for (i = 0; i < ARRAY_SIZE(argtypes); i++) {
+		Hjava_lang_Class* sigclass;
+		sigclass = getClassFromSignaturePart(METHOD_ARG_TYPE(mth, i), mth->class->loader, &info);
+		if (sigclass == 0) {
+			discardErrorInfo(&info);
+			return 0;
+		}
+		if (sigclass !=
+			(struct Hjava_lang_Class *)OBJARRAY_DATA(argtypes)[i]) {
+			return (0);
+		}
+	}
+
+	return (1);
+}
+
+
+static
+Hjava_lang_reflect_Method*
+findMatchingMethod(struct Hjava_lang_Class* clas, 
+		   struct Hjava_lang_String* name, 
+		   HArrayOfObject* arr, jboolean declared)
+{
+	Method* mth = CLASS_METHODS(clas);
+	int n = CLASS_NMETHODS(clas);
+	int i;
+	for (i = 0;  i < n;  ++mth, ++i) {
+		if (((mth->accflags & ACC_PUBLIC) || declared)
+		    && utf8ConstEqualJavaString(mth->name, name)) {
+			if (checkParameters(mth, arr))
+				return (makeMethod(clas, i));
+		}
+	}
+	return (0);
 }
 
 Hjava_lang_reflect_Method*
-java_lang_Class_getMethod0(struct Hjava_lang_Class* this, struct Hjava_lang_String* name, HArrayOfObject* arr, jint declared)
+java_lang_Class_getMethod0(struct Hjava_lang_Class* this, struct Hjava_lang_String* name, HArrayOfObject* arr, jboolean declared)
 {
 	Hjava_lang_Class* clas;
+	Hjava_lang_reflect_Method *rmeth;
 
 	clas = this;
 	do {
-		Method* mth = CLASS_METHODS(clas);
-		int n = CLASS_NMETHODS(clas);
-		int i;
-		for (i = 0;  i < n;  ++mth, ++i) {
-			if (((mth->accflags & ACC_PUBLIC) || declared)
-			    && equalUtf8JavaStrings (mth->name, name)) {
-				if (checkParameters(mth, arr))
-					return (makeMethod(clas, i));
-			}
+		rmeth = findMatchingMethod(clas, name, arr, declared);
+		if (rmeth != 0) {
+			return (rmeth);
 		}
 		clas = clas->superclass;
 	} while (!declared && clas != NULL);
 
+	/* If the class is an interface, check implemented interfaces as well.
+	 * Those are the interfaces this interface inherited
+	 */
+	if (CLASS_IS_INTERFACE(this)) {
+		int i;
+		for (i = 0; i < this->total_interface_len; i++) {
+			rmeth = findMatchingMethod(this->interfaces[i], 
+						   name, arr, declared);
+			if (rmeth != 0) {
+				return (rmeth);
+			}
+		}
+	}
+
 	/* like SignalError, except that the name of the class that is
 	 * not found becomes the error message 
 	 */
-	throwException(execute_java_constructor(
-		"java.lang.NoSuchMethodException", 0, 
+	throwException((struct Hjava_lang_Throwable*)execute_java_constructor(
+		"java.lang.NoSuchMethodException", 0, 0, 
 		"(Ljava/lang/String;)V", name));
 }
 
 struct Hjava_lang_reflect_Constructor*
-java_lang_Class_getConstructor0(struct Hjava_lang_Class* this, HArrayOfObject* arr, jint declared) 
+java_lang_Class_getConstructor0(struct Hjava_lang_Class* this, HArrayOfObject* arr, jboolean declared) 
 {
 	Hjava_lang_Class* clas = this;
 
@@ -724,11 +866,14 @@ java_lang_Class_getConstructor0(struct Hjava_lang_Class* this, HArrayOfObject* a
 	SignalError("java.lang.NoSuchMethodException", ""); /* FIXME */
 }
 
+static
 Hjava_lang_reflect_Field*
-java_lang_Class_getField0(struct Hjava_lang_Class* clazz, struct Hjava_lang_String* name, int declared)
+checkForField(struct Hjava_lang_Class* clazz, struct Hjava_lang_String* name, jboolean declared)
 {
+	int i;
 	Hjava_lang_Class* clas;
 
+	/* first try this class's own or inherited fields */
 	clas = (Hjava_lang_Class*) clazz;
 	do {
 		Field* fld = CLASS_FIELDS(clas);
@@ -736,11 +881,31 @@ java_lang_Class_getField0(struct Hjava_lang_Class* clazz, struct Hjava_lang_Stri
 		int i;
 		for (i = 0;  i < n;  ++fld, ++i) {
 			if (((fld->accflags & ACC_PUBLIC) || declared)
-			    && equalUtf8JavaStrings (fld->name, name)) {
+			    && utf8ConstEqualJavaString(fld->name, name)) {
 				return makeField(clas, i);
 			}
 		}
 		clas = clas->superclass;
 	} while (!declared && clas != NULL);
+
+	/* then try this class's interfaces fields using the wonders of
+	 * recursion.  */
+	for (i = 0; i < clazz->total_interface_len; i++) {
+		Hjava_lang_reflect_Field *f;
+		f = checkForField(clazz->interfaces[i], name, declared);
+		if (f != 0) {
+			return (f);
+		}
+	}
+	return (0);
+}
+
+Hjava_lang_reflect_Field*
+java_lang_Class_getField0(struct Hjava_lang_Class* clazz, struct Hjava_lang_String* name, jboolean declared)
+{
+	Hjava_lang_reflect_Field* f = checkForField(clazz, name, declared);
+	if (f != 0) {
+		return (f);
+	}
 	SignalError("java.lang.NoSuchFieldException", ""); /* FIXME */
 }
