@@ -4,7 +4,7 @@
  * Copyright (c) 1996, 1997
  *	Transvirtual Technologies, Inc.  All rights reserved.
  *
- * See the file "lib-license.terms" for information on usage and redistribution 
+ * See the file "license.terms" for information on usage and redistribution 
  * of this file. 
  */
 
@@ -18,16 +18,18 @@
 #include "../../../kaffe/kaffevm/classMethod.h"
 #include "../../../kaffe/kaffevm/itypes.h"
 #include "../../../kaffe/kaffevm/support.h"
+#include "../../../kaffe/kaffevm/soft.h"
+#include "../../../kaffe/kaffevm/exception.h"
 #include "../../../kaffe/kaffevm/baseClasses.h"
-#include "Method.h"
-#include "Boolean.h"
-#include "Byte.h"
-#include "Character.h"
-#include "Short.h"
-#include "Integer.h"
-#include "Long.h"
-#include "Float.h"
-#include "Double.h"
+#include "java_lang_reflect_Method.h"
+#include "java_lang_Boolean.h"
+#include "java_lang_Byte.h"
+#include "java_lang_Character.h"
+#include "java_lang_Short.h"
+#include "java_lang_Integer.h"
+#include "java_lang_Long.h"
+#include "java_lang_Float.h"
+#include "java_lang_Double.h"
 #include <native.h>
 #include "jni.h"
 #include "defs.h"
@@ -47,7 +49,7 @@ java_lang_reflect_Method_getModifiers(struct Hjava_lang_reflect_Method* this)
 }
 
 jobject
-Java_java_lang_reflect_Method_invoke(JNIEnv* env, jobject _this, jobject _obj, jarray _argobj)
+Java_java_lang_reflect_Method_invoke(JNIEnv* env, jobject _this, jobject _obj, jobjectArray _argobj)
 {
 	Hjava_lang_Class* clazz;
 	Hjava_lang_Object* robj;
@@ -55,13 +57,13 @@ Java_java_lang_reflect_Method_invoke(JNIEnv* env, jobject _this, jobject _obj, j
 	jint slot;
 	jvalue args[255]; /* should this be allocated dynamically? */
 	jvalue ret;
-	Hjava_lang_Class* a;
+	Hjava_lang_Class *a, *t;
 	Hjava_lang_Object** body;
 	int i;
 	int j;
 	int len;
-	char* sig;
 	char rettype;
+	errorInfo info;
 
 	/* Bit of a hack this */
 	Hjava_lang_Object* obj = (Hjava_lang_Object*)_obj;
@@ -72,11 +74,14 @@ Java_java_lang_reflect_Method_invoke(JNIEnv* env, jobject _this, jobject _obj, j
 	clazz = unhand(this)->clazz;
 
 	/* 
-	 * make sure constants are resolved and static initializers are run 
-	 * before invoking a method on this class
+	 * A method invocation via reflection is a first active use, so
+	 * initialize the class object.  (XXX: check whether that's true.)
 	 */
-	if (clazz->state != CSTATE_OK) {
-		processClass(clazz, CSTATE_OK);
+	/* XXX use JNI here XXX */
+	if (clazz->state < CSTATE_USABLE) {
+		if (processClass(clazz, CSTATE_COMPLETE, &info) == false) {
+			throwError(&info);
+		}
 	}
 
 	slot = unhand(this)->slot;
@@ -89,23 +94,32 @@ Java_java_lang_reflect_Method_invoke(JNIEnv* env, jobject _this, jobject _obj, j
 	meth = &clazz->methods[slot];
 
 	len = argobj ? obj_length(argobj) : 0;
-	body = argobj ? (Hjava_lang_Object**)unhand(argobj)->body : 0;
-	for (sig = meth->signature->data + 1, i = j = 0;
-	     *sig != ')' && i < len; ++sig, ++i, ++j) {
+
+	if (len != METHOD_NARGS(meth)) {
+		SignalError("java.lang.IllegalArgumentException", "wrong number of arguments");
+	}
+
+	body = argobj ? OBJARRAY_DATA(argobj) : 0;
+	for (i = j = 0; i < len; ++i, ++j) {
+		t = getClassFromSignaturePart(METHOD_ARG_TYPE(meth, i),
+					  meth->class->loader, &info);
+		if (t == 0) {
+			throwError(&info);
+		}
 		a = body[i] ? OBJECT_CLASS(body[i]) : 0;
-		switch (*sig) {
+		if (CLASS_IS_PRIMITIVE(t)) switch (CLASS_PRIM_SIG(t)) {
 		case 'Z':
 			if (a == javaLangBooleanClass) {
 				args[j].z = unhand((Hjava_lang_Boolean*)body[i])->value;
 			} else {
-				SignalError("java.lang.IllegalArgumentException", "");
+				SignalError("java.lang.IllegalArgumentException", "expecting boolean value");
 			}
 			break;
 		case 'B':
 			if (a == javaLangByteClass) {
 				args[j].b = unhand((Hjava_lang_Byte*)body[i])->value;
 			} else {
-				SignalError("java.lang.IllegalArgumentException", "");
+				SignalError("java.lang.IllegalArgumentException", "expecting byte value");
 			}
 			break;
 		case 'C':
@@ -115,7 +129,7 @@ Java_java_lang_reflect_Method_invoke(JNIEnv* env, jobject _this, jobject _obj, j
 			else if (a == javaLangByteClass) {
 				args[j].c = unhand((Hjava_lang_Byte*)body[i])->value;
 			} else {
-				SignalError("java.lang.IllegalArgumentException", "");
+				SignalError("java.lang.IllegalArgumentException", "expecting character value");
 			}
 			break;
 		case 'S':
@@ -125,7 +139,7 @@ Java_java_lang_reflect_Method_invoke(JNIEnv* env, jobject _this, jobject _obj, j
 			else if (a == javaLangByteClass) {
 				args[j].s = unhand((Hjava_lang_Byte*)body[i])->value;
 			} else {
-				SignalError("java.lang.IllegalArgumentException", "");
+				SignalError("java.lang.IllegalArgumentException", "expecting short value");
 			}
 			break;
 		case 'I':
@@ -141,7 +155,7 @@ Java_java_lang_reflect_Method_invoke(JNIEnv* env, jobject _this, jobject _obj, j
 			else if (a == javaLangByteClass) {
 				args[j].i = unhand((Hjava_lang_Byte*)body[i])->value;
 			} else {
-				SignalError("java.lang.IllegalArgumentException", "");
+				SignalError("java.lang.IllegalArgumentException", "expecting int value");
 			}
 			break;
 		case 'J':
@@ -160,7 +174,7 @@ Java_java_lang_reflect_Method_invoke(JNIEnv* env, jobject _this, jobject _obj, j
 			else if (a == javaLangByteClass) {
 				args[j].j = unhand((Hjava_lang_Byte*)body[i])->value;
 			} else {
-				SignalError("java.lang.IllegalArgumentException", "");
+				SignalError("java.lang.IllegalArgumentException", "expecting long value");
 			}
 			++j;
 			break;
@@ -183,7 +197,7 @@ Java_java_lang_reflect_Method_invoke(JNIEnv* env, jobject _this, jobject _obj, j
 			else if (a == javaLangByteClass) {
 				args[j].f = unhand((Hjava_lang_Byte*)body[i])->value;
 			} else {
-				SignalError("java.lang.IllegalArgumentException", "");
+				SignalError("java.lang.IllegalArgumentException", "expecting float value");
 			}
 			break;
 		case 'D':
@@ -208,29 +222,21 @@ Java_java_lang_reflect_Method_invoke(JNIEnv* env, jobject _this, jobject _obj, j
 			else if (a == javaLangByteClass) {
 				args[j].d = unhand((Hjava_lang_Byte*)body[i])->value;
 			} else {
-				SignalError("java.lang.IllegalArgumentException", "");
+				SignalError("java.lang.IllegalArgumentException", "expecting double value");
 			}
 			++j;
 			break;
-		case '[':
-			while (*++sig == '[')
-				;
-			if (*sig == 'L')
-				/* fall through */
-		case 'L':
-			while (*++sig != ';')
-				;
-			args[j].l = body[i];
-			break;
 		default:
 			ABORT();
+		} else if (body[i] == 0 ||
+			   instanceof(t, OBJECT_CLASS(body[i]))) {
+			args[j].l = body[i];
+		} else {
+			SignalError("java.lang.IllegalArgumentException", "incompatible object types");
 		}
 	}
-	if (*sig != ')' || i < len) {
-		SignalError("java.lang.IllegalArgumentException", "");
-	}
 
-	rettype = *++sig;
+	rettype = *METHOD_RET_TYPE(meth);
 
 	/* Select which method to really call, and call it */
 	if (METHOD_IS_STATIC(meth)) {	/* static method */
@@ -245,14 +251,13 @@ Java_java_lang_reflect_Method_invoke(JNIEnv* env, jobject _this, jobject _obj, j
 		case 'J': ret.j = CallStaticTypeMethodA(Long); break;
 		case 'F': ret.f = CallStaticTypeMethodA(Float); break;
 		case 'D': ret.d = CallStaticTypeMethodA(Double); break;
-		case 'I': ret.j = CallStaticTypeMethodA(Int); break;
+		case 'I': ret.i = CallStaticTypeMethodA(Int); break;
 		case 'S': ret.s = CallStaticTypeMethodA(Short); break;
 		case 'B': ret.b = CallStaticTypeMethodA(Byte); break;
 		case 'Z': ret.z = CallStaticTypeMethodA(Boolean); break;
 		case 'C': ret.c = CallStaticTypeMethodA(Char); break;
-		case 'L': 
+		case 'L':
 		case '[': ret.l = CallStaticTypeMethodA(Object); break;
-
 #undef CallStaticTypeMethodA
 		default:
 			ABORT();
@@ -263,7 +268,6 @@ Java_java_lang_reflect_Method_invoke(JNIEnv* env, jobject _this, jobject _obj, j
 		 * This if applies if we are called from Constructor.newInstance
 		 */
 		ret.l = (*env)->NewObjectA(env, clazz, meth, args);
-
 		/* override return type parsed from signature */
 		rettype = 'L';
 	}
@@ -305,13 +309,13 @@ Java_java_lang_reflect_Method_invoke(JNIEnv* env, jobject _this, jobject _obj, j
 	 */
 	targetexc = (*env)->ExceptionOccurred(env);
 	if (targetexc != 0) {
-		Hjava_lang_Object* obj;
+		struct Hjava_lang_Throwable* e;
 
 		(*env)->ExceptionClear(env);
-		obj = execute_java_constructor(
-			"java.lang.reflect.InvocationTargetException", 0, 
+		e = (struct Hjava_lang_Throwable *)execute_java_constructor(
+			"java.lang.reflect.InvocationTargetException", 0, 0,
 			"(Ljava/lang/Throwable;)V", targetexc);
-		throwException(obj);
+		throwException(e);
 		assert(!"Not here");
 	}
 
@@ -320,34 +324,35 @@ Java_java_lang_reflect_Method_invoke(JNIEnv* env, jobject _this, jobject _obj, j
 		robj = 0;
 		break;
 	case 'J':
-		robj = execute_java_constructor(0, javaLangLongClass, "(J)V", ret.j);
+		robj = execute_java_constructor(0, 0, javaLangLongClass, "(J)V", ret.j);
 		break;
 	case 'F':
-		robj = execute_java_constructor(0, javaLangFloatClass, "(F)V", ret.f);
+		robj = execute_java_constructor(0, 0, javaLangFloatClass, "(F)V", ret.f);
 		break;
 	case 'D':
-		robj = execute_java_constructor(0, javaLangDoubleClass, "(D)V", ret.d);
+		robj = execute_java_constructor(0, 0, javaLangDoubleClass, "(D)V", ret.d);
 		break;
 	case 'I':
-		robj = execute_java_constructor(0, javaLangIntegerClass, "(I)V", ret.i);
+		robj = execute_java_constructor(0, 0, javaLangIntegerClass, "(I)V", ret.i);
 		break;
 	case 'S':
-		robj = execute_java_constructor(0, javaLangShortClass, "(S)V", ret.s);
+		robj = execute_java_constructor(0, 0, javaLangShortClass, "(S)V", ret.s);
 		break;
 	case 'B':
-		robj = execute_java_constructor(0, javaLangByteClass, "(B)V", ret.b);
+		robj = execute_java_constructor(0, 0, javaLangByteClass, "(B)V", ret.b);
 		break;
 	case 'Z':
-		robj = execute_java_constructor(0, javaLangBooleanClass, "(Z)V", ret.z);
+		robj = execute_java_constructor(0, 0, javaLangBooleanClass, "(Z)V", ret.z);
 		break;
 	case 'C':
-		robj = execute_java_constructor(0, javaLangCharacterClass, "(C)V", ret.c);
+		robj = execute_java_constructor(0, 0, javaLangCharacterClass, "(C)V", ret.c);
 		break;
 	case 'L':
 	case '[':
 		robj = ret.l;
 		break;
 	default:
+		robj = 0;	/* avoid compiler warning */
 		ABORT();
 	}
 
