@@ -4,7 +4,7 @@
  * Copyright (c) 1996, 1997
  *	Transvirtual Technologies, Inc.  All rights reserved.
  *
- * See the file "lib-license.terms" for information on usage and redistribution 
+ * See the file "license.terms" for information on usage and redistribution 
  * of this file. 
  */
 
@@ -17,7 +17,9 @@
 #include "../../../kaffe/kaffevm/object.h"
 #include "../../../kaffe/kaffevm/constants.h"
 #include "../../../kaffe/kaffevm/access.h"
-#include <native.h>
+#include "../../../kaffe/kaffevm/soft.h"
+#include "../../../kaffe/kaffevm/baseClasses.h"
+#include "java_lang_Object.h"
 
 /*
  * Generate object hash code.
@@ -64,19 +66,12 @@ java_lang_Object_clone(struct Hjava_lang_Object* o)
 {
 	Hjava_lang_Object* obj;
 	Hjava_lang_Class* class;
-	static Hjava_lang_Class* cloneClass;
-	int i;
-
-	/* * Lookup cloneable class interface if we don't have it  */
-	if (cloneClass == 0) {
-		cloneClass = lookupClass("java/lang/Cloneable");
-	}
 
 	class = OBJECT_CLASS(o);
 
 	if (!CLASS_IS_ARRAY(class)) {
 		/* Check class is cloneable and throw exception if it isn't */
-		if (soft_instanceof(cloneClass, o) == 0) {
+		if (soft_instanceof(CloneClass, o) == 0) {
 			SignalError("java.lang.CloneNotSupportedException", class->name->data);
 		}
 		/* Clone an object */
@@ -96,7 +91,7 @@ java_lang_Object_clone(struct Hjava_lang_Object* o)
  * Wait for this object to be notified.
  */
 void
-java_lang_Object_wait(struct Hjava_lang_Object* o, jlong timeout)
+java_lang_Object_wait0(struct Hjava_lang_Object* o, jlong timeout)
 {
 	waitCond(o, timeout);
 }
