@@ -65,8 +65,8 @@ private static String escape(String str, boolean isKey) {
 			sb.append("\\r");
 			break;
 		default:
-			if (ch > 0x7e) {
-				sb.append("\\\\u");
+			if (ch < 0x20 || ch > 0x7e) {
+				sb.append("\\u");
 				sb.append(Character.forDigit((ch >> 12) & 0xf, 16));
 				sb.append(Character.forDigit((ch >> 8) & 0xf, 16));
 				sb.append(Character.forDigit((ch >> 4) & 0xf, 16));
@@ -120,24 +120,24 @@ public void list(PrintWriter out) {
 }
 
 public Enumeration propertyNames() {
-	final Vector result = new Vector();
+	SortedSet nameSet = new TreeSet();	// sort keys just for fun
 
 	// Add main properties
 	for (Enumeration e = keys(); e.hasMoreElements(); ) {
-		result.addElement(e.nextElement());
+		nameSet.add(e.nextElement());
 	}
 
 	// Add non-overridden default properties
 	if (defaults != null) {
 		for (Enumeration e = defaults.keys(); e.hasMoreElements(); ) {
 			Object def = e.nextElement();
-			if (!result.contains(def))
-				result.addElement(def);
+			if (!nameSet.contains(def))
+				nameSet.add(def);
 		}
 	}
 
 	// Return enumeration of vector
-	return result.elements();
+	return Collections.enumeration(nameSet);
 }
 
 public synchronized void load(InputStream in) throws IOException {
@@ -260,18 +260,18 @@ private static int getEscapedChar(PushbackInputStream in) throws IOException {
 			case '\n':
 				break;
 			}				// fall through
-		case '\n':
+		case '\n':				// line continuation
 			while ((ch = in.read()) != -1 && ch <= ' ');
-			return ch;
+			in.unread(ch);
+			return getEscapedChar(in);
 		case 'n':
 			return '\n';
 		case 'r':
 			return '\r';
 		case 't':
 			return '\t';
-		case '\\':
-			switch ((ch = in.read())) {
-			case 'u':
+		case 'u':
+			{
 				int[] dig = new int[4];
 				int n;
 			    getUnicode:
@@ -289,10 +289,8 @@ private static int getEscapedChar(PushbackInputStream in) throws IOException {
 				}
 				// not unreachable, break getUnicode go here
 				while (n > 0)
-				    in.unread(dig[--n]);	// fall through
-			default:
-				in.unread(ch);
-				return '\\';
+				    in.unread(dig[--n]);
+				return 'u';
 			}
 
 		// not fall through, previous switch always return
